package db

import (
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectDouShenImGroups 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `DouShenImGroupFindByParams` 方法
func BatchSelectDouShenImGroups(Db *xorm.Engine, params map[string]interface{}) (*[]model.DouShenImGroup, error) {
	var DouShenImGroupData []model.DouShenImGroup
	if err := Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&DouShenImGroupData); err != nil {
		return nil, logx.Warn(err)
	}
	return &DouShenImGroupData, nil
}

// DouShenImGroupInsert 插入单条数据
func DouShenImGroupInsert(Db *xorm.Engine, DouShenImGroup *model.DouShenImGroup) (int, error) {
	_, err := Db.InsertOne(DouShenImGroup)
	if err != nil {
		return 0, err
	}
	return DouShenImGroup.Id, nil
}

// BatchAddDouShenImGroups 批量新增数据
func BatchAddDouShenImGroups(Db *xorm.Engine, DouShenImGroupData []*model.DouShenImGroup) (int64, error) {
	affected, err := Db.Insert(DouShenImGroupData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetDouShenImGroupCount(Db *xorm.Engine) int {
	var DouShenImGroup model.DouShenImGroup
	session := Db.Where("")
	count, err := session.Count(&DouShenImGroup)
	if err != nil {
		return 0
	}
	return int(count)
}

// DouShenImGroupDelete 删除记录
func DouShenImGroupDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.DouShenImGroup{})
	} else {
		return Db.Where("id = ?", id).Delete(model.DouShenImGroup{})
	}
}

// DouShenImGroupUpdate 更新记录
func DouShenImGroupUpdate(Db *xorm.Engine, id interface{}, DouShenImGroup *model.DouShenImGroup, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = Db.Where("id=?", id).Cols(forceColums...).Update(DouShenImGroup)
	} else {
		affected, err = Db.Where("id=?", id).Update(DouShenImGroup)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// DouShenImGroupGetOneByParams 通过传入的参数查询数据(单条)
func DouShenImGroupGetOneByParams(Db *xorm.Engine, kind int, params map[string]interface{}) (*model.DouShenImGroup, error) {
	var m model.DouShenImGroup
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).And("kind = ?", kind).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

func DouShenImGroupGetOneByParamsForOfficial(Db *xorm.Engine, kind int) (*model.DouShenImGroup, error) {
	var m model.DouShenImGroup
	has, err := Db.Where("kind = ?", kind).Get(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return &m, nil
}

func DouShenImGroupGetOneByParamsForFans(Db *xorm.Engine, kind int, uid int64) (*model.DouShenImGroup, error) {
	var m model.DouShenImGroup
	has, err := Db.Where("kind = ?", kind).And("uid =?", uid).Get(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return &m, nil
}

func DouShenImGroupFindByParamsByKind(Db *xorm.Engine, kind int, params map[string]interface{}) (*[]model.DouShenImGroup, error) {
	var m []model.DouShenImGroup
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).And("kind =?", kind).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).And("kind =?", kind).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}
	}
}

// DouShenImGroupFindByParams 通过传入的参数查询数据(多条)
func DouShenImGroupFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.DouShenImGroup, error) {
	var m []model.DouShenImGroup
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}
	}
}
