package db

import (
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectMallOrdItems 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `MallOrdItemFindByParams` 方法
func BatchSelectMallOrdItems(Db *xorm.Engine, params map[string]interface{}) (*[]model.MallOrdItem, error) {
	var MallOrdItemData []model.MallOrdItem
	if err := Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&MallOrdItemData); err != nil {
		return nil, logx.Warn(err)
	}
	return &MallOrdItemData, nil
}

// MallOrdItemInsert 插入单条数据
func MallOrdItemInsert(Db *xorm.Engine, MallOrdItem *model.MallOrdItem) (int64, error) {
	_, err := Db.InsertOne(MallOrdItem)
	if err != nil {
		return 0, err
	}
	return MallOrdItem.OrdId, nil
}

// BatchAddMallOrdItems 批量新增数据
func BatchAddMallOrdItems(Db *xorm.Engine, MallOrdItemData []*model.MallOrdItem) (int64, error) {
	affected, err := Db.Insert(MallOrdItemData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetMallOrdItemCount(Db *xorm.Engine) int {
	var MallOrdItem model.MallOrdItem
	session := Db.Where("")
	count, err := session.Count(&MallOrdItem)
	if err != nil {
		return 0
	}
	return int(count)
}

// MallOrdItemDelete 删除记录
func MallOrdItemDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.MallOrdItem{})
	} else {
		return Db.Where("id = ?", id).Delete(model.MallOrdItem{})
	}
}

// MallOrdItemUpdate 更新记录
func MallOrdItemUpdate(Db *xorm.Engine, id interface{}, MallOrdItem *model.MallOrdItem, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = Db.Where("id=?", id).Cols(forceColums...).Update(MallOrdItem)
	} else {
		affected, err = Db.Where("id=?", id).Update(MallOrdItem)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// MallOrdItemGetOneByParams 通过传入的参数查询数据(单条)
func MallOrdItemGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.MallOrdItem, error) {
	var m model.MallOrdItem
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// MallOrdItemFindByParams 通过传入的参数查询数据(多条)
func MallOrdItemFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.MallOrdItem, error) {
	var m []model.MallOrdItem
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}
