package db

import (
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectMasterAreaVisitsFlows 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `MasterAreaVisitsFlowsFindByParams` 方法
func BatchSelectMasterAreaVisitsFlows(Db *xorm.Engine, params map[string]interface{}) (*[]model.MasterAreaVisitsFlows, error) {
	var MasterAreaVisitsFlowsData []model.MasterAreaVisitsFlows
	if err := Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&MasterAreaVisitsFlowsData); err != nil {
		return nil, logx.Warn(err)
	}
	return &MasterAreaVisitsFlowsData, nil
}

// MasterAreaVisitsFlowsInsert 插入单条数据
func MasterAreaVisitsFlowsInsert(Db *xorm.Engine, MasterAreaVisitsFlows *model.MasterAreaVisitsFlows) (int, error) {
	_, err := Db.InsertOne(MasterAreaVisitsFlows)
	if err != nil {
		return 0, err
	}
	return MasterAreaVisitsFlows.Id, nil
}

// BatchAddMasterAreaVisitsFlows 批量新增数据
func BatchAddMasterAreaVisitsFlows(Db *xorm.Engine, MasterAreaVisitsFlowsData []*model.MasterAreaVisitsFlows) (int64, error) {
	affected, err := Db.Insert(MasterAreaVisitsFlowsData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetMasterAreaVisitsFlowsCount(Db *xorm.Engine) int {
	var MasterAreaVisitsFlows model.MasterAreaVisitsFlows
	session := Db.Where("")
	count, err := session.Count(&MasterAreaVisitsFlows)
	if err != nil {
		return 0
	}
	return int(count)
}

// MasterAreaVisitsFlowsDelete 删除记录
func MasterAreaVisitsFlowsDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.MasterAreaVisitsFlows{})
	} else {
		return Db.Where("id = ?", id).Delete(model.MasterAreaVisitsFlows{})
	}
}

// MasterAreaVisitsFlowsUpdate 更新记录
func MasterAreaVisitsFlowsUpdate(Db *xorm.Engine, id interface{}, MasterAreaVisitsFlows *model.MasterAreaVisitsFlows, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = Db.Where("id=?", id).Cols(forceColums...).Update(MasterAreaVisitsFlows)
	} else {
		affected, err = Db.Where("id=?", id).Update(MasterAreaVisitsFlows)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// MasterAreaVisitsFlowsGetOneByParams 通过传入的参数查询数据(单条)
func MasterAreaVisitsFlowsGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.MasterAreaVisitsFlows, error) {
	var m model.MasterAreaVisitsFlows
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// MasterAreaVisitsFlowsFindByParams 通过传入的参数查询数据(多条)
func MasterAreaVisitsFlowsFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.MasterAreaVisitsFlows, error) {
	var m []model.MasterAreaVisitsFlows
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}
