package db

import (
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectRegionalAgentUserBelongs 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `RegionalAgentUserBelongFindByParams` 方法
func BatchSelectRegionalAgentUserBelongs(Db *xorm.Engine, params map[string]interface{}) (*[]model.RegionalAgentUserBelong, error) {
	var RegionalAgentUserBelongData []model.RegionalAgentUserBelong
	if err := Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&RegionalAgentUserBelongData); err != nil {
		return nil, logx.Warn(err)
	}
	return &RegionalAgentUserBelongData, nil
}

// RegionalAgentUserBelongInsert 插入单条数据
func RegionalAgentUserBelongInsert(Db *xorm.Engine, RegionalAgentUserBelong *model.RegionalAgentUserBelong) (int64, error) {
	_, err := Db.InsertOne(RegionalAgentUserBelong)
	if err != nil {
		return 0, err
	}
	return RegionalAgentUserBelong.Id, nil
}

// BatchAddRegionalAgentUserBelongs 批量新增数据
func BatchAddRegionalAgentUserBelongs(Db *xorm.Engine, RegionalAgentUserBelongData []*model.RegionalAgentUserBelong) (int64, error) {
	affected, err := Db.Insert(RegionalAgentUserBelongData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetRegionalAgentUserBelongCount(Db *xorm.Engine) int {
	var RegionalAgentUserBelong model.RegionalAgentUserBelong
	session := Db.Where("")
	count, err := session.Count(&RegionalAgentUserBelong)
	if err != nil {
		return 0
	}
	return int(count)
}

// RegionalAgentUserBelongDelete 删除记录
func RegionalAgentUserBelongDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.RegionalAgentUserBelong{})
	} else {
		return Db.Where("id = ?", id).Delete(model.RegionalAgentUserBelong{})
	}
}

// RegionalAgentUserBelongUpdate 更新记录
func RegionalAgentUserBelongUpdate(Db *xorm.Engine, id interface{}, RegionalAgentUserBelong *model.RegionalAgentUserBelong, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = Db.Where("id=?", id).Cols(forceColums...).Update(RegionalAgentUserBelong)
	} else {
		affected, err = Db.Where("id=?", id).Update(RegionalAgentUserBelong)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// RegionalAgentUserBelongGetOneByParams 通过传入的参数查询数据(单条)
func RegionalAgentUserBelongGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.RegionalAgentUserBelong, error) {
	var m model.RegionalAgentUserBelong
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// RegionalAgentUserBelongFindByParams 通过传入的参数查询数据(多条)
func RegionalAgentUserBelongFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.RegionalAgentUserBelong, error) {
	var m []model.RegionalAgentUserBelong
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}
