package db

import (
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"xorm.io/xorm"
)

func GetUserVirtualAmountFlow(eg *xorm.Engine, args map[string]string) *[]model.UserVirtualCoinFlow {
	var data []model.UserVirtualCoinFlow
	sess := eg.Where("uid=?", args["uid"])
	if args["date"] != "" {
		sess.And("date=?", args["date"])
	}
	if args["coin_id"] != "" {
		sess.And("coin_id=?", args["coin_id"])
	}
	if args["type"] != "" {
		sess.And("transfer_type=?", args["type"])
	}
	err := sess.Limit(utils.StrToInt(args["p"]), (utils.StrToInt(args["p"])-1)*utils.StrToInt(args["size"])).OrderBy("id desc").Find(&data)
	if err != nil {
		return nil
	}
	return &data
}
func GetUserVirtualAmountOne(session *xorm.Session, uid int, coinId int) (*model.UserVirtualAmount, error) {

	var m model.UserVirtualAmount
	isExist, err := session.Table("user_virtual_amount").Where("uid = ? AND coin_id = ?", uid, coinId).Get(&m)
	if err != nil {
		return nil, err
	}
	if !isExist {
		return nil, nil
	}
	return &m, nil

}
func UserVirtualAmountFindByIdWithSession(session *xorm.Session, uid, coinId int) (*model.UserVirtualAmount, error) {
	var m model.UserVirtualAmount
	has, err := session.Where("uid = ? AND coin_id = ?", uid, coinId).Get(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	if has == false {
		m.Amount = "0"
		m.CoinId = coinId
		m.Uid = uid
		one, err := session.InsertOne(&m)
		if err != nil || one == 0 {
			return nil, logx.Warn(err)
		}
	}
	return &m, nil
}

func UserVirtualAmountFindById(sess *xorm.Session, uid, coinId int) (*model.UserVirtualAmount, error) {
	var m model.UserVirtualAmount
	has, err := sess.Where("uid = ? AND coin_id = ?", uid, coinId).Get(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	if has == false {
		m.Amount = "0"
		m.CoinId = coinId
		m.Uid = uid
		one, err := sess.InsertOne(&m)
		if err != nil || one == 0 {
			return nil, logx.Warn(err)
		}
	}
	return &m, nil
}
func UserVirtualAmountUpdate(sess *xorm.Session, uid, coinId interface{}, userVirtualAmount *model.UserVirtualAmount, forceCols ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceCols != nil {
		affected, err = sess.Where("uid = ? AND coin_id = ?", uid, coinId).Cols(forceCols...).Update(userVirtualAmount)
	} else {
		affected, err = sess.Where("uid = ? AND coin_id = ?", uid, coinId).Update(userVirtualAmount)
	}
	if err != nil {
		return 0, logx.Warn(err)
	}
	return affected, nil
}
func UserVirtualCoinFlowInsertOne(sess *xorm.Session, m *model.UserVirtualCoinFlow) error {
	_, err := sess.InsertOne(m)
	if err != nil {
		return err
	}
	return nil
}

// 在事务中更新用户信息
func UserVirtualAmountUpdateWithSession(session *xorm.Session, uid, coinId interface{}, userVirtualAmount *model.UserVirtualAmount, forceCols ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceCols != nil {
		affected, err = session.Where("uid = ? AND coin_id = ?", uid, coinId).Cols(forceCols...).Update(userVirtualAmount)
	} else {
		affected, err = session.Where("uid = ? AND coin_id = ?", uid, coinId).Update(userVirtualAmount)
	}
	if err != nil {
		return 0, logx.Warn(err)
	}
	return affected, nil
}

// 在事务中更新用户信息
func UserVirtualFlowUpdate(eg *xorm.Engine, id int, userVirtualCoinFlow *model.UserVirtualCoinFlow, forceCols ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceCols != nil {
		affected, err = eg.Where("id = ? ", id).Cols(forceCols...).Update(userVirtualCoinFlow)
	} else {
		affected, err = eg.Where("id = ? ", id).Update(userVirtualCoinFlow)
	}
	if err != nil {
		return 0, logx.Warn(err)
	}
	return affected, nil
}

// UserVirtualCoinFlowInsertOneWithSession 在事务中使用，插入一条流水记录
func UserVirtualCoinFlowInsertOneWithSession(session *xorm.Session, m *model.UserVirtualCoinFlow) error {
	_, err := session.InsertOne(m)
	if err != nil {
		return err
	}
	return nil
}
