package db

import (
	"applet/app/db/model"
	"applet/app/utils/logx"

	"xorm.io/xorm"
)

//SessionUserWithDrawApplyInsertOne 事务插入一条提现记录单
func SessionUserWithDrawApplyInsertOne(Db *xorm.Session, m *model.FinWithdrawApply) (int64, error) {
	affected, err := Db.InsertOne(m)
	if err != nil {

		return 0, err
	}
	return affected, nil
}

//UserWithDrawApplyInsertOne 插入一条提现记录单
func UserWithDrawApplyInsertOne(Db *xorm.Engine, m *model.FinWithdrawApply) (int64, error) {
	affected, err := Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

//UserWithDrawApplyInsertOne 插入一条提现记录单
func UserWithDrawApplyWithSessionInsertOne(session *xorm.Session, m *model.FinWithdrawApply) (int64, error) {
	affected, err := session.InsertOne(m)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

//UserWithDrawApplyDeletedOne is 删除指定id的数据
func UserWithDrawApplyDeletedOne(Db *xorm.Engine, id interface{}) (int64, error) {
	m := new(model.FinWithdrawApply)
	affected, err := Db.ID(id).Delete(m)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// UserWithDrawApplyByUID is 获取支出记录
func UserWithDrawApplyByUID(Db *xorm.Engine, uid interface{}, time string, limit, start int) ([]*model.FinWithdrawApply, error) {
	var m []*model.FinWithdrawApply
	if err := Db.Where("uid = ? AND create_at like ?", uid, time+"%").Desc("create_at").Limit(limit, start).Find(&m); err != nil {
		return nil, logx.Warn(err)
	}
	return m, nil
}

// UserWithDrawApplyByUIDByTime is 获取某时间段所有的提现申请单号 审核失败的不要
func UserWithDrawApplyByUIDByTime(Db *xorm.Engine, uid interface{}, stime, etime string) ([]*model.FinWithdrawApply, error) {
	var m []*model.FinWithdrawApply
	if err := Db.Where("uid = ? AND state!=3 AND create_at > ? AND create_at < ?", uid, stime, etime).Find(&m); err != nil {
		return nil, logx.Warn(err)
	}
	return m, nil
}

func UserWithDrawApplyByUIDById(Db *xorm.Engine, id int64) (*model.FinWithdrawApply, error) {
	var m model.FinWithdrawApply
	has, err := Db.Where("id = ?", id).Get(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	if !has {
		return nil, logx.Error("Not found")
	}
	return &m, nil
}

//UserWithDrawApplySumByState is 统计某种状态的提现订单的金额
func UserWithDrawApplySumByState(Db *xorm.Engine, uid interface{}, state ...interface{}) (float64, error) {
	m := new(model.FinWithdrawApply)
	total, err := Db.Where("uid = ?", uid).In("state", state...).Sum(m, "amount")
	if err != nil {
		return 0, logx.Warn(err)
	}
	return total, nil
}

// UserWithDrawApplyDeleteByUserDelete is 用户删除时的删除
func UserWithDrawApplyDeleteByUserDelete(Db *xorm.Engine, uid interface{}) (int64, error) {
	return Db.Where("uid = ?", uid).Delete(model.FinWithdrawApply{})
}
