package model

import (
	"time"
)

type MallOrd struct {
	OrdId                   int64     `json:"ord_id" xorm:"not null pk BIGINT(20)"`
	MainOrdId               int64     `json:"main_ord_id" xorm:"not null comment('主订单号') index BIGINT(20)"`
	Uid                     int       `json:"uid" xorm:"comment('用户id') index INT(11)"`
	MoneyInAccount          int       `json:"money_in_account" xorm:"comment('') INT(11)"`
	BuyerName               string    `json:"buyer_name" xorm:"comment('购买人') VARCHAR(255)"`
	AppId                   string    `json:"app_id" xorm:"comment('购买人') VARCHAR(255)"`
	BuyerPhone              string    `json:"buyer_phone" xorm:"comment('购买人手机号') VARCHAR(255)"`
	CostPrice               string    `json:"cost_price" xorm:"comment('价格') DECIMAL(12,2)"`
	MaocaoPrice             string    `json:"maocao_price" xorm:"comment('汇率') DECIMAL(12,2)"`
	DeductCoin              string    `json:"deduct_coin" xorm:"comment('抵扣券') DECIMAL(12,2)"`
	CostVirtualCoin         string    `json:"cost_virtual_coin" xorm:"comment('消耗的虚拟币') DECIMAL(12,2)"`
	CoinPrice               string    `json:"coin_price" xorm:"comment('消耗的虚拟币') DECIMAL(12,2)"`
	VirtualCoinId           int       `json:"virtual_coin_id" xorm:"comment('使用的虚拟币id') INT(11)"`
	State                   int       `json:"state" xorm:"comment('订单状态：0未支付 1已支付 2已发货 3已完成 4售后中 5部分售后中 6关闭') TINYINT(1)"`
	PayTime                 time.Time `json:"pay_time" xorm:"comment('支付时间') DATETIME"`
	PayChannel              int       `json:"pay_channel" xorm:"not null comment('支付方式：1balance 2alipay 3wx_pay 4澳门工银支付 5澳门工银支付-支付宝支付 6澳门工银支付-微信支付 7') TINYINT(1)"`
	ShippingTime            time.Time `json:"shipping_time" xorm:"comment('发货时间') DATETIME"`
	LogisticCompany         string    `json:"logistic_company" xorm:"not null default '' comment('物流公司') VARCHAR(255)"`
	LogisticNum             string    `json:"logistic_num" xorm:"not null default '' comment('物流单号') VARCHAR(255)"`
	ReceiverPhone           string    `json:"receiver_phone" xorm:"not null default '' comment('收货人手机号') VARCHAR(20)"`
	ReceiverName            string    `json:"receiver_name" xorm:"not null default '' comment('收货人名字') VARCHAR(255)"`
	ReceiverAddressDetail   string    `json:"receiver_address_detail" xorm:"not null default '' comment('收货人地址') VARCHAR(255)"`
	ShippingType            int       `json:"shipping_type" xorm:"not null default 1 comment('运送方式：1快递送货') TINYINT(1)"`
	CouponDiscount          string    `json:"coupon_discount" xorm:"not null default 0.00 comment('优惠券折扣额') DECIMAL(12,2)"`
	DiscountPrice           string    `json:"discount_price" xorm:"not null default 0.00 comment('立减') DECIMAL(12,2)"`
	UserCouponId            int64     `json:"user_coupon_id" xorm:"comment('使用的优惠券id') BIGINT(20)"`
	ReturnInsuranceFee      string    `json:"return_insurance_fee" xorm:"not null default 0.00 comment('退货无忧费用') DECIMAL(12,2)"`
	IsReceipt               int       `json:"is_receipt" xorm:"not null default 0 comment('是否开具发票 0否 1是') TINYINT(255)"`
	ShippingFee             string    `json:"shipping_fee" xorm:"not null default 0.00 comment('运费') DECIMAL(12,2)"`
	Comment                 string    `json:"comment" xorm:"not null comment('备注') VARCHAR(2048)"`
	ProvinceName            string    `json:"province_name" xorm:"not null default '' comment('收货省份') VARCHAR(255)"`
	CityName                string    `json:"city_name" xorm:"not null default '' comment('收货城市') VARCHAR(255)"`
	CountyName              string    `json:"county_name" xorm:"not null default '' comment('收货区域') VARCHAR(255)"`
	PayNum                  string    `json:"pay_num" xorm:"not null default '' comment('交易流水') VARCHAR(255)"`
	ConfirmTime             time.Time `json:"confirm_time" xorm:"comment('确认时间') DATETIME"`
	EstimateIntegral        string    `json:"estimate_integral" xorm:"not null default 0.0000 comment('预计积分') DECIMAL(12,4)"`
	EstimateCommission      string    `json:"estimate_commission" xorm:"not null default 0.0000 comment('预计佣金') DECIMAL(12,4)"`
	CreateTime              time.Time `json:"create_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	TeamCheckTime           time.Time `json:"team_check_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('') DATETIME"`
	IsTeamCheck             int       `json:"is_team_check" xorm:"comment('商品id') INT(11)"`
	UpdateTime              time.Time `json:"update_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	DeletedTime             time.Time `json:"deleted_time" xorm:"deleted comment('删除时间') DATETIME"`
	FinishTime              time.Time `json:"finish_time" xorm:"comment('完成时间') DATETIME"`
	OrderType               int       `json:"order_type" xorm:"not null default 1 comment('订单类型：1普通订单 2拼团订单') TINYINT(3)"`
	Data                    string    `json:"data" xorm:"comment('订单相关的数据') TEXT"`
	GroupBuyCommission      string    `json:"group_buy_commission" xorm:"default 0.0000 comment('团购未中奖佣金') DECIMAL(12,4)"`
	GroupBuySettleTime      time.Time `json:"group_buy_settle_time" xorm:"comment('拼团结算时间') DATETIME"`
	SettleTime              time.Time `json:"settle_time" xorm:"comment('结算时间') DATETIME"`
	GroupBuyCommissionTime  time.Time `json:"group_buy_commission_time" xorm:"comment('拼团分佣时间') DATETIME"`
	CommissionTime          time.Time `json:"commission_time" xorm:"comment('分佣时间') DATETIME"`
	ShareUid                int       `json:"share_uid" xorm:"comment('分享人') INT(11)"`
	GoodsType               int       `json:"goods_type" xorm:"comment('商品活动类型') INT(11)"`
	IsConsign               int       `json:"is_consign" xorm:"comment('是否是超级拼团寄售0否1是') TINYINT(1)"`
	UserLevelData           string    `json:"user_level_data" xorm:"comment('会员礼包相关') TEXT"`
	GoodsId                 int       `json:"goods_id" xorm:"comment('商品id') INT(11)"`
	UserLevel               int       `json:"user_level" xorm:"comment('升级礼包升级等级') INT(11)"`
	IsHasUserLevel          int       `json:"is_has_user_level" xorm:"comment('是否有会员权益') INT(1)"`
	IsGiveUserLevel         int       `json:"is_give_user_level" xorm:"comment('是否赠送升级礼包') INT(1)"`
	SupplierMerchantId      int       `json:"supplier_merchant_id" xorm:"comment('供应商id') INT(11)"`
	SupplierOrdId           string    `json:"supplier_ord_id" xorm:"comment('供应商订单id') VARCHAR(255)"`
	SupplierCloudChainOrdId string    `json:"supplier_cloud_chain_ord_id" xorm:"comment('供应商订单1688订单id') VARCHAR(255)"`
	SupplierStoreOrdId      string    `json:"supplier_store_ord_id" xorm:"comment('供应链店铺订单号') VARCHAR(255)"`
	Pvd                     string    `json:"pvd" xorm:"comment('') VARCHAR(255)"`
	IsSubsidyEnd            int       `json:"is_subsidy_end" xorm:"comment('') INT(11)"`
	PayOnBehalfUid          int       `json:"pay_on_behalf_uid" xorm:"comment('0') INT(11)"`
	ProvinceId              string    `json:"province_id" xorm:"comment('省id') CHAR(12)"`
	CityId                  string    `json:"city_id" xorm:"comment('市id') CHAR(12)"`
	CountyId                string    `json:"county_id" xorm:"comment('县/区') CHAR(11)"`
	CurrencyCode            string    `json:"currency_code" xorm:"default 'CNY' comment('货币符号') VARCHAR(10)"`
	TotalShippingFee        string    `json:"total_shipping_fee" xorm:"default '0.00' comment('供应链同一商家总运费') DECIMAL(12,2)"`
	StoreOrdId              int64     `json:"store_ord_id" xorm:"not null default '0' comment('店铺主订单号') BIGINT(20)"`
	TransactionId           string    `json:"transaction_id" xorm:"comment('交易号') VARCHAR(255)"`
	IcbcIntegral            string    `json:"icbc_integral" xorm:"comment('澳门工行积分') DECIMAL(12,4)"`
	ParentUid               int       `json:"parent_uid" xorm:"comment('推荐人') INT(11)"`
	RewardCoinAmount        string    `json:"reward_coin_amount" xorm:"comment('') DECIMAL(12,4)"`
	IsVirtualGoods          int       `json:"is_virtual_goods" xorm:"default 0 comment('') INT(1)"`
	RewardCoinId            int       `json:"reward_coin_id" xorm:"default 0 comment('') INT(11)"`
	RewardVirtualUserLv     string    `json:"reward_virtual_user_lv" xorm:"not null comment('自定义参数json') TEXT"`
	BaseCommission          string    `json:"base_commission" xorm:"comment('积分兑换模式基数') DECIMAL(12,4)"`
	PlatformCostPrice       string    `json:"platform_cost_price" xorm:"comment('平台成本价') DECIMAL(12,4)"`
	IsSettle                int       `json:"is_settle" xorm:"default 0 comment('') INT(11)"`
	PointId                 string    `json:"point_id" xorm:"comment('澳门自提点id') VARCHAR(20)"`
	PickUpId                string    `json:"pick_up_id" xorm:"comment('自提点id') VARCHAR(20)"`
	EndTime                 int       `json:"end_time" xorm:"comment('') INT(11)"`
	ConsignForSaleUid       int       `json:"consign_for_sale_uid" xorm:"comment('') INT(11)"`
	ConsignForSaleOid       string    `json:"consign_for_sale_oid" xorm:"comment('') VARCHAR(20)"`
	PickUpUid               int       `json:"pick_up_uid" xorm:"comment('自提点id') INT(11)"`
	AddressType             string    `json:"address_type" xorm:"comment('') VARCHAR(255)"`
	MacaoAddress            string    `json:"macao_address" xorm:"comment('澳门世宝的珠海地址') TEXT"`
	VirtualGoodsInfo        string    `json:"virtual_goods_info" xorm:"not null default '' comment('') VARCHAR(1024)"`
	PayWay                  string    `json:"pay_way" xorm:" default '' comment('') VARCHAR(255)"`
	ErrMsg                  string    `json:"err_msg" xorm:" default '' comment('') VARCHAR(255)"`
	ParamData               string    `json:"param_data" xorm:" default '' comment('') VARCHAR(255)"`
	Platform                string    `json:"platform" xorm:" default '' comment('') VARCHAR(255)"`
	PayDiscountMoney        string    `json:"pay_discount_money" xorm:"default '0.00' comment('供应链同一商家总运费') DECIMAL(12,2)"`
	IsDeductDiscount        int       `json:"is_deduct_discount" xorm:"comment('') INT(11)"`
}
