package md

// 0已付款,1已收货,2成功,3供应商已结算,4已失效,5平台已结算
const (
	OrderStatePaid = iota
	OrderStateRecv
	OrderStateSucc
	OrderStatePvdSettle
	OrderStateFail
	OrderStateSysSettle
)

const (
	OrderTypeSelf             = iota     // 自购订单
	OrderTypeShare                       // 分享订单
	OrderTypeFree                        // 免单订单
	OrderTypeActSelf                     //活动自购订单
	OrderTypeWechatActSelf               //小程序活动自购订单
	OrderTypeWechatActShare              //小程序活动分享订单
	OrderTypeTljFree                     //淘礼金免单订单
	OrderTypeTljRelationFree             //淘礼金渠道免单订单
	OrderTypeWechatActSelfNew = iota + 1 //小程序活动自购订单

)

var OrderState = map[int]struct{}{
	OrderStatePaid:      {},
	OrderStateRecv:      {},
	OrderStateSucc:      {},
	OrderStatePvdSettle: {},
	OrderStateFail:      {},
	OrderStateSysSettle: {},
}

type OrderDetail struct {
	GoodsInfo     OrderList    `json:"goods_info"`
	OrderInfoList []InfoDetail `json:"order_info_list"`
}
type InfoDetail struct {
	Title   string `json:"title"`
	Content string `json:"content"`
	Type    string `json:"type"`
}
type OrderStates struct {
	Name string `json:"name"`
	Type string `json:"type"`
}
type OrderStatusNew struct {
	Name             string `json:"name"`
	Type             string `json:"type"`
	Tips             string `json:"tips"`
	TipsReplaceKey   string `json:"tips_replace_key"`
	TipsReplaceColor string `json:"tips_replace_color"`
	AmountText       string `json:"amount_text"`
	IncomeText       string `json:"income_text"`
}
type OrderList struct {
	Uid                 int          `json:"uid"`
	Nickname            string       `json:"nickname"`
	HidOrder            bool         `json:"hid_order"`
	OrdId               string       `json:"ord_id"`
	ProviderOid         string       `json:"provider_oid"`
	ItemId              string       `json:"item_id"`
	ItemNum             int          `json:"item_num"`
	ItemTitle           string       `json:"item_title"`
	ItemPrice           string       `json:"item_price"`
	Provider            string       `json:"provider"`
	PaidPrice           string       `json:"paid_price"`
	OrderType           int          `json:"order_type"`
	PriceType           int          `json:"price_type"`
	PriceName           string       `json:"price_name"`
	UserCommissionStr   string       `json:"user_commission_str"`
	UserCommission      string       `json:"user_commission"`
	OldUserCommission   string       `json:"old_user_commission"`
	FreeCommission      string       `json:"free_commission"`
	UnionCommission     string       `json:"union_commission"`
	UserCommissionRate  string       `json:"user_commission_rate"`
	Reason              string       `json:"reason"`
	State               int          `json:"state"`
	OldState            int          `json:"old_state"`
	IsFastSign          int          `json:"is_fast_sign"`
	OrderTypes          string       `json:"order_types"`
	UserAlipayName      string       `json:"user_alipay_name"`
	UserAlipayLogo      string       `json:"user_alipay_logo"`
	CreateAt            string       `json:"create_at"`
	UpdateAt            string       `json:"update_at"`
	ConfirmAt           string       `json:"confirm_at"`
	SettleAt            string       `json:"settle_at"`
	PvdSettleAt         string       `json:"pvd_settle_at"`
	Thumbnail           string       `json:"thumbnail"`
	Info                string       `json:"info"`
	UpgradeInfo         string       `json:"upgrade_info"`
	UpgradeReq          string       `json:"upgrade_req"`
	UpgradeValue        string       `json:"upgrade_value"`
	SettleInfo          string       `json:"settle_info"`
	OrderCompare        string       `json:"order_compare"`
	StationOrdId        string       `json:"station_ord_id"`
	CoinList            []CoinList   `json:"coin_list"`
	ReturnMoney         string       `json:"return_money"`
	OrderDetailViewType string       `json:"order_detail_view_type"`
	OrderInfoList       []InfoDetail `json:"order_info_list"`
	StateStr            string       `json:"state_str"`
	AmountText          string       `json:"amount_text"`
	OrdModeType         string       `json:"ord_mode_type"`
	AvatarUrl           string       `json:"avatar_url"`
	UserLevel           string       `json:"user_level"`
	UserLevelName       string       `json:"user_level_name"`
}
type CoinList struct {
	CoinId    string `json:"coin_id"`
	CoinName  string `json:"coin_name"`
	CoinValue string `json:"coin_value"`
	CoinBili  string `json:"coin_bili"`
}
type OrderInfoList struct {
	Title string `json:"title"`
	Type  string `json:"type"`
}
type CommissionFirstParam struct {
	CommissionParam  CommissionParam `json:"commission_param"`
	Uid              string          `json:"uid"`
	IsShare          int             `json:"is_share"`
	OldLv            string          `json:"old_lv"` //升级礼包读取的是升级前的等级
	NewLv            string          `json:"new_lv"` //升级礼包读取的是升级后的等级
	Provider         string          `json:"provider"`
	IsAllLevelReturn int             `json:"is_all_level_return"` // 是否返回所有层级
	GoodsId          string          `json:"goods_id,omitempty"`  // 用于标记是哪个商品的
	OwnbuyReturnType int             `json:"ownbuy_return_type"`  //自购是否返利 0返利 1不返利
	Oid              string          `json:"oid"`
	ShowLevel        string          `json:"show_level"`
	IsShowExtend     string          `json:"is_show_extend"`
}
type CommissionParam struct {
	GoodsPrice           string `json:"goods_price"`
	OldPrice             string `json:"old_price"`
	PaidPrice            string `json:"paid_price"`
	Commission           string `json:"commission"`
	CommissionRate       string `json:"commission_rate"`
	CouponPrice          string `json:"coupon_price"`
	WlGoodsPrice         string `json:"wl_goods_price"` //卷后价
	LowerPrice           string `json:"lower_price"`
	LowestCouponPrice    string `json:"lowestCouponPrice"`
	MinGroupPrice        string `json:"min_group_price"`
	TikTokTeamCommission string `json:"tik_tok_team_commission"`
	IsTikTokTeamOrder    string `json:"is_tik_tok_team_order"`
	DouyinBili           string `json:"douyin_bili"`
}
type PrivacyCfg struct {
	IsOn           string `json:"is_on"`
	Nickname       string `json:"nickname"`
	Phone          string `json:"phone"`
	GoodsTitle     string `json:"goods_title"`
	GoodsImage     string `json:"goods_image"`
	OrderNum       string `json:"order_num"`
	SelfCommission string `json:"self_commission"`
	SonCommission  string `json:"son_commission"`
}
type OrderItemList struct {
	ItemId         string `json:"item_id"`
	ItemNum        int    `json:"item_num"`
	Price          string `json:"price"`
	Commission     string `json:"commission"`
	CommissionRate string `json:"commission_rate"`
}
type UserPid struct {
	Pid              string
	Uid              int
	OrderType        int
	Lv               int
	NewLv            int
	OwnbuyReturnType int //0有返利 1没有返利
}

type OrderInfo struct {
	Oid                      int64
	PvdOid                   string
	ParentOrdId              int64
	Pvd                      string
	ItemId                   string
	ItemNum                  int
	ItemPrice                float64
	ItemTitle                string
	ItemLink                 string  // 用户ID
	Thumbnail                string  // 缩略图
	PaidPrice                float64 // 付款总价
	OrderType                int     // 0自购,1分享订单,2新人免单
	Pid                      string  // 推广位ID
	IsCompare                int     // 是否比价订单, 只有拼多多和淘宝有
	Commission               float64 // 总佣金
	CommissionRate           float64 // 佣金比例
	Reason                   string  // 撤单原因
	PayTime                  int64   // 支付时间
	StartTime                int64   // 下单时间
	EndTime                  int64   // 结束时间
	State                    int     // 订单状态, 0已付款,1已收货,2成功,3已结算,4已失效
	ConfirmAt                int64   //确认收货时间
	CheckSuccessAt           int64   //审核通过时间
	CostPrice                float64
	Source                   string //
	IsTikTokTeamOrder        int
	TikTokTeamCommission     string
	TikTokTeamRealCommission string
	OptPvd                   string
}

type OrderFindRequest struct {
	Oid string `json:"oid"`
	Uid string `json:"uid"`
}
type CardOrderInfo struct {
	ID             string `json:"id"`
	OrderID        string `json:"orderId"`
	CreateTime     string `json:"create_time"`
	UID            string `json:"uid"`
	Zuid           string `json:"zuid"`
	Statut         string `json:"statut"`
	Account        string `json:"account"`
	BuyNum         string `json:"buy_num"`
	GID            string `json:"g_id"`
	SpecID         string `json:"spec_id"`
	Type           string `json:"type"`
	Payment        string `json:"payment"`
	Msg            string `json:"msg"`
	PayTime        string `json:"pay_time"`
	LID            string `json:"l_id"`
	SysOrderID     string `json:"sys_orderId"`
	Status         string `json:"status"`
	RechargeStatus string `json:"recharge_status"`
	IsToreturn     string `json:"is_toreturn"`
	CardMsg        string `json:"card_msg"`
	IsFh           string `json:"is_fh"`
	Oid            string `json:"oid"`
}
type VipOrder struct {
	ID               string `json:"id"`
	ShareAppExtendID string `json:"share_app_extend_id"`
	Oid              string `json:"oid"`
	Gid              string `json:"gid"`
	Info             string `json:"info"`
	Commission       string `json:"commission"`
	CommissionRate   string `json:"commission_rate"`
	UID              string `json:"uid"`
	Payment          string `json:"payment"`
	Status           string `json:"status"`
	ChannelTag       string `json:"channelTag"`
	NewCustomer      string `json:"newCustomer"`
	CreateTime       string `json:"create_time"`
	SignTime         string `json:"sign_time"`
	JsTime           string `json:"js_time"`
	ZmUpdateTime     string `json:"zm_update_time"`
	StationPid       string `json:"station_pid"`
}

// 定制订单列表的站长
const CustomizedOrderListMaster = "[68823769], [22255132]"
