package svc

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"time"
	"xorm.io/xorm"
)

func UpdateUserFinValidAndInterFlow(engine *xorm.Engine, money, Title, ordType string, types, orderAction, uid, id int, ordId, otherId int64) error {
	session := engine.NewSession()
	defer session.Close()
	session.Begin()
	userProfile, err := db.UserProfileFindByIdWithSession(session, uid)
	if err != nil || userProfile == nil {
		_ = session.Rollback()
		if err == nil {
			err = errors.New("获取用户余额信息失败")
		}
		return err
	}
	beforeAmount := userProfile.FinValid
	if types == 0 {
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) + utils.StrToFloat64(money))
	} else if types == 1 {
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) - utils.StrToFloat64(money))
	}
	afterAmount := userProfile.FinValid
	userProfile.FinTotal = userProfile.FinTotal + utils.StrToFloat32(money)
	affected, err := db.UserProfileUpdateWithSession(session, uid, userProfile, "fin_valid,fin_total")
	if err != nil || affected == 0 {
		_ = session.Rollback()
		if err == nil {
			err = errors.New("更新用户余额信息失败")
		}
		return err
	}
	err = flowInsertSess(session, uid, money, orderAction, ordId, otherId, id, Title, ordType, types, beforeAmount, afterAmount)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	return session.Commit()
}
func UpdateUserFinValidAndInterFlowSess(session *xorm.Session, money, Title, ordType string, types, orderAction, uid, id int, ordId, otherId int64) error {
	userProfile, err := db.UserProfileFindByIdWithSession(session, uid)
	if err != nil || userProfile == nil {
		if err == nil {
			err = errors.New("获取用户余额信息失败")
		}
		return err
	}
	beforeAmount := userProfile.FinValid
	if types == 0 {
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) + utils.StrToFloat64(money))
	} else if types == 1 {
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) - utils.StrToFloat64(money))
	}
	afterAmount := userProfile.FinValid
	userProfile.FinTotal = userProfile.FinTotal + utils.StrToFloat32(money)
	affected, err := db.UserProfileUpdateWithSession(session, uid, userProfile, "fin_valid,fin_total")
	if err != nil || affected == 0 {
		if err == nil {
			err = errors.New("更新用户余额信息失败")
		}
		return err
	}
	err = flowInsertSess(session, uid, money, orderAction, ordId, otherId, id, Title, ordType, types, beforeAmount, afterAmount)
	if err != nil {
		return err
	}
	return nil
}

func flowInsertSess(session *xorm.Session, uid int, paidPrice string, orderAction int, ordId int64, id int64, goodsId int, ItemTitle string, ordType string, types int, beforeAmount string, afterAmount string) error {
	now := time.Now()
	if err := db.FinUserFlowInsertOneWithSession(
		session,
		&model.FinUserFlow{
			Type:         types,
			Uid:          uid,
			Amount:       paidPrice,
			BeforeAmount: beforeAmount,
			AfterAmount:  afterAmount,
			OrdType:      ordType,
			OrdId:        utils.Int64ToStr(ordId),
			OrdAction:    orderAction,
			OrdDetail:    utils.IntToStr(goodsId),
			State:        2,
			OtherId:      id,
			OrdTitle:     ItemTitle,
			OrdTime:      int(now.Unix()),
			CreateAt:     now,
			UpdateAt:     now,
		}); err != nil {
		_ = logx.Warn(err)
		return err
	}
	return nil
}
