package svc

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"time"
	"xorm.io/xorm"
)

func ExchangeUserVirFinValidAndInterFlowWithSession(session *xorm.Session, money float64, Title, fee string, types, transferType, uid, coinId, coinIdTo int, ordId int64, coinTransferJson string, transferId, toUid int64) (int64, error) {
	UserVirtualAmount, err := db.UserVirtualAmountFindByIdWithSession(session, uid, coinId)
	if err != nil || UserVirtualAmount == nil {
		if err == nil {
			err = errors.New("获取用户余额信息失败")
		}
		return 0, err
	}
	beforeAmount := UserVirtualAmount.Amount
	afterAmount := ""
	if types == 1 {
		afterAmount = utils.Float64ToStrByPrec(utils.AnyToFloat64(UserVirtualAmount.Amount)+money, 6)
		UserVirtualAmount.Amount = utils.GetPrec(utils.Float64ToStrByPrec(utils.AnyToFloat64(UserVirtualAmount.Amount)+money, 5), "4")
	} else if types == 2 {
		afterAmount = utils.Float64ToStrByPrec(utils.AnyToFloat64(UserVirtualAmount.Amount)-money, 6)
		UserVirtualAmount.Amount = utils.GetPrec(utils.Float64ToStrByPrec(utils.AnyToFloat64(UserVirtualAmount.Amount)-money, 5), "4")
		if utils.StrToInt64(UserVirtualAmount.Amount) < 0 {
			_ = session.Rollback()
			err = errors.New("余额不足")
			return 0, err
		}

	}
	affected, err := db.UserVirtualAmountUpdateWithSession(session, uid, coinId, UserVirtualAmount, "amount")
	if err != nil || affected == 0 {
		if err == nil {
			err = errors.New("更新用户余额信息失败")
		}
		return 0, err
	}
	id, err := virtualCoinFlowInsert(session, uid, coinId, coinIdTo, utils.Float64ToStrByPrec(money, 8), fee, ordId, Title, types, transferType, beforeAmount, afterAmount, coinTransferJson, transferId, toUid)
	if err != nil {
		return 0, err
	}
	return id, nil
}

func virtualCoinFlowInsert(session *xorm.Session, uid, coinId, coinIdTo int, money, SysFee string, ordId int64, ItemTitle string, types, transferType int, beforeAmount string, afterAmount, coinTransferJson string, transferId, toUid int64) (int64, error) {
	now := time.Now()
	data := &model.UserVirtualCoinFlow{
		Uid:              uid,
		OrdId:            utils.Int64ToStr(ordId),
		CoinId:           coinId,
		CoinIdTo:         coinIdTo,
		Direction:        types,
		Title:            ItemTitle,
		Amout:            money,
		BeforeAmout:      beforeAmount,
		AfterAmout:       afterAmount,
		SysFee:           SysFee,
		CreateTime:       now,
		TransferType:     transferType,
		CoinTransferData: coinTransferJson,
		TransferId:       int(transferId),
		ToUid:            int(toUid),
		Date:             now.Format("2006-01"),
	}
	if err := db.UserVirtualCoinFlowInsertOneWithSession(
		session, data); err != nil {
		_ = logx.Warn(err)
		return 0, err
	}
	return data.Id, nil
}

func UpdateUserFinValidAndInterFlowWithSession(session *xorm.Session, money, Title, ordType string, types, orderAction, uid, id int, ordId, otherId int64) error {
	if utils.StrToFloat64(money) <= 0 {
		return nil
	}
	userProfile, err := db.UserProfileFindByIdWithSession(session, uid)
	if err != nil || userProfile == nil {
		_ = session.Rollback()
		if err == nil {
			err = errors.New("获取用户余额信息失败")
		}
		return err
	}
	beforeAmount := userProfile.FinValid
	if types == 0 {
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) + utils.StrToFloat64(money))
	} else if types == 1 {
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) - utils.StrToFloat64(money))
	}
	afterAmount := userProfile.FinValid
	userProfile.FinTotal = userProfile.FinTotal + utils.StrToFloat32(money)
	affected, err := db.UserProfileUpdateWithSession(session, uid, userProfile, "fin_valid,fin_total")
	if err != nil || affected == 0 {
		if err == nil {
			err = errors.New("更新用户余额信息失败")
		}
		return err
	}
	err = flowInsert(session, uid, money, orderAction, ordId, otherId, id, Title, ordType, types, beforeAmount, afterAmount)
	if err != nil {
		return err
	}
	return nil
}

// 开始写入流水
//uid:用户id，paidPrice金额，ordId订单id，id其他关联订单，具体根据订单类型判断，goodsId（OrdDetail记录商品ID或提现账号），ItemTitle订单标题
//type:0收入,1支出，beforeAmount变更前金额，afterAmount变更后
//orderAction:10自购,11推广,12团队,13免单，20提现,21消费,22退款,23拼团返佣，24资金池分红
//ordType:订单类型taobao,jd,pdd,vip,suning,kaola,own自营,withdraw提现,vip_refund会员升级退款,vip_order会员升级余额支付流水,group_buy拼团
func flowInsert(session *xorm.Session, uid int, paidPrice string, orderAction int, ordId int64, id int64, goodsId int, ItemTitle string, ordType string, types int, beforeAmount string, afterAmount string) error {
	now := time.Now()
	if err := db.FinUserFlowInsertOneWithSession(
		session,
		&model.FinUserFlow{
			Type:         types,
			Uid:          uid,
			Amount:       paidPrice,
			BeforeAmount: beforeAmount,
			AfterAmount:  afterAmount,
			OrdType:      ordType,
			OrdId:        utils.Int64ToStr(ordId),
			OrdAction:    orderAction,
			OrdDetail:    utils.IntToStr(goodsId),
			State:        2,
			OtherId:      id,
			OrdTitle:     ItemTitle,
			OrdTime:      int(now.Unix()),
			CreateAt:     now,
			UpdateAt:     now,
		}); err != nil {
		_ = logx.Warn(err)
		return err
	}
	return nil
}
