package db

import (
	commDb "applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	"errors"
	"fmt"
	"github.com/gin-gonic/gin"
	"reflect"
	"xorm.io/xorm"
)

type CloudIssuanceGoodsDb struct {
	Db *xorm.Engine `json:"model"`
}

func (cloudIssuanceGoodsDb *CloudIssuanceGoodsDb) Set(c *gin.Context) { // set方法
	cloudIssuanceGoodsDb.Db = commDb.DBs[c.GetString("mid")]
}

func (cloudIssuanceGoodsDb *CloudIssuanceGoodsDb) GetCloudIssuanceGood(goodsId interface{}) (m *model.CloudIssuanceGoods, err error) {
	m = new(model.CloudIssuanceGoods)
	has, err := cloudIssuanceGoodsDb.Db.Where("goods_id = ?", goodsId).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceGoodsDb *CloudIssuanceGoodsDb) FindCloudIssuanceGoods(params map[string]interface{}) (m []*model.CloudIssuanceGoods, err error) {
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := cloudIssuanceGoodsDb.Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := cloudIssuanceGoodsDb.Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := cloudIssuanceGoodsDb.Db.Where(query, params["value"]).OrderBy("id ASC").Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return m, nil
		}
	}
}

func (cloudIssuanceGoodsDb *CloudIssuanceGoodsDb) GetCloudIssuanceGoodByCategory(categoryId int) (m *model.CloudIssuanceGoods, err error) {
	m = new(model.CloudIssuanceGoods)
	has, err := cloudIssuanceGoodsDb.Db.Where("cloud_issuance_category_id = ?", categoryId).OrderBy("id ASC").Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceGoodsDb *CloudIssuanceGoodsDb) DeleteCloudIssuanceGoods(id interface{}) (affected int64, err error) {
	utils.FilePutContents("DeleteCloudIssuanceGoods", utils.SerializeStr(map[string]interface{}{
		"id": id,
	}))
	_, err = cloudIssuanceGoodsDb.Db.Where("id=?", id).Delete(model.CloudIssuanceGoods{})
	if err != nil {
		return 0, err
	}
	return 1, nil
}
