package db

import (
	commDb "applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	"errors"
	"fmt"
	"github.com/gin-gonic/gin"
	"reflect"
	"xorm.io/xorm"
)

type CloudIssuanceGoodsCategoryDb struct {
	Db *xorm.Engine `json:"model"`
}

func (cloudIssuanceGoodsCategoryDb *CloudIssuanceGoodsCategoryDb) Set(c *gin.Context) { // set方法
	cloudIssuanceGoodsCategoryDb.Db = commDb.DBs[c.GetString("mid")]
}

func (cloudIssuanceGoodsCategoryDb *CloudIssuanceGoodsCategoryDb) GetCloudIssuanceGoodsCategory(id int) (m *model.CloudIssuanceGoodsCategory, err error) {
	m = new(model.CloudIssuanceGoodsCategory)
	has, err := cloudIssuanceGoodsCategoryDb.Db.Where("id = ?", id).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceGoodsCategoryDb *CloudIssuanceGoodsCategoryDb) GetDefaultCloudIssuanceGoodsCategory() (m *model.CloudIssuanceGoodsCategory, err error) {
	m = new(model.CloudIssuanceGoodsCategory)
	has, err := cloudIssuanceGoodsCategoryDb.Db.OrderBy("sort DESC").Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到数据源分类")
	}
	return m, nil
}

func (cloudIssuanceGoodsCategoryDb *CloudIssuanceGoodsCategoryDb) FindCloudIssuanceGoodsCategory(params map[string]interface{}) (m []*model.CloudIssuanceGoodsCategory, err error) {
	if params["key"] == nil {
		//查询全部数据
		err := cloudIssuanceGoodsCategoryDb.Db.OrderBy("sort DESC").Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := cloudIssuanceGoodsCategoryDb.Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := cloudIssuanceGoodsCategoryDb.Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return m, nil
		}
	}
}
