package db

import (
	commDb "applet/app/db"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	"applet/cloud_issuance/md"
	"github.com/gin-gonic/gin"
	"xorm.io/xorm"
)

type CloudIssuancePackageOrdDb struct {
	Db *xorm.Engine `json:"model"`
}

func (cloudIssuancePackageOrdDb *CloudIssuancePackageOrdDb) Set(c *gin.Context) { // set方法
	cloudIssuancePackageOrdDb.Db = commDb.DBs[c.GetString("mid")]
}

func (cloudIssuancePackageOrdDb *CloudIssuancePackageOrdDb) GetCloudIssuancePackageOrd(ordNo string) (m *model.CloudIssuancePackageOrd, err error) {
	m = new(model.CloudIssuancePackageOrd)
	has, err := cloudIssuancePackageOrdDb.Db.Where("ord_no =?", ordNo).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuancePackageOrdDb *CloudIssuancePackageOrdDb) GetCloudIssuancePackageOrdByUid(uid interface{}) (m *model.CloudIssuancePackageOrd, err error) {
	m = new(model.CloudIssuancePackageOrd)
	has, err := cloudIssuancePackageOrdDb.Db.Where("uid =?", uid).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuancePackageOrdDb *CloudIssuancePackageOrdDb) InsertCloudIssuancePackageOrd(m *model.CloudIssuancePackageOrd) (id int, err error) {
	_, err = cloudIssuancePackageOrdDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	id = m.Id
	return id, nil
}

func (cloudIssuancePackageOrdDb *CloudIssuancePackageOrdDb) SaveCloudIssuancePackageOrd(id interface{}, m *model.CloudIssuancePackageOrd, forceColums ...string) (affected int64, err error) {
	if forceColums != nil {
		affected, err = cloudIssuancePackageOrdDb.Db.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = cloudIssuancePackageOrdDb.Db.Where("id=?", id).Update(m)
	}
	return
}

func (cloudIssuancePackageOrdDb *CloudIssuancePackageOrdDb) UserOrdList(uid interface{}, req md.BuyPackageRecordsRequest) (resp []*md.BuyPackageRecordsResponse, total int64, err error) {
	total, err = cloudIssuancePackageOrdDb.Db.Table("cloud_issuance_package_ord").
		Join("INNER", "cloud_issuance_robot_package", "cloud_issuance_package_ord.package_id = cloud_issuance_robot_package.id").
		Where("cloud_issuance_package_ord.uid = ?", uid).
		OrderBy("cloud_issuance_package_ord.id DESC").
		Cols("cloud_issuance_package_ord.*, cloud_issuance_robot_package.name as package_name").
		Limit(req.Limit, (req.Page-1)*req.Limit).FindAndCount(&resp)
	return
}
