package db

import (
	commDb "applet/app/db"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	db "applet/cloud_issuance/db/official"
	model2 "applet/cloud_issuance/db/official/model"
	"applet/cloud_issuance/enum"
	"github.com/gin-gonic/gin"
	"strconv"
	"time"
	"xorm.io/xorm"
)

type CloudIssuanceUserRobotDb struct {
	Db       *xorm.Engine `json:"model"`
	Uid      int          `json:"uid"`
	MasterId string       `json:"master_id"`
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) Set(c *gin.Context, uid int) { // set方法
	cloudIssuanceUserRobotDb.Db = commDb.DBs[c.GetString("mid")]
	cloudIssuanceUserRobotDb.Uid = uid
	cloudIssuanceUserRobotDb.MasterId = c.GetString("mid")
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) GetCloudIssuanceUserRobot() (m *model.CloudIssuanceUserRobot, err error) {
	m = new(model.CloudIssuanceUserRobot)
	has, err := cloudIssuanceUserRobotDb.Db.Where("uid = ?", cloudIssuanceUserRobotDb.Uid).
		And("is_enabled = 0").Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) GetCloudIssuanceUserFreeRobot() (m *model.CloudIssuanceUserRobot, err error) {
	//TODO::先将所有已过期的状态改变
	timeStamp := time.Now().Unix()
	sql := "update cloud_issuance_user_robot set is_enabled = ? where end_time <= ? and robot_kind =?"
	_, err = cloudIssuanceUserRobotDb.Db.Exec(sql, 3, timeStamp, enum.ShareRobot)
	if err != nil {
		return nil, logx.Error(err)
	}

	m = new(model.CloudIssuanceUserRobot)
	has, err := cloudIssuanceUserRobotDb.Db.Where("uid = -1").
		And("robot_kind =?", enum.ShareRobot).
		And("is_enabled = 0").Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) InsertCloudIssuanceUserRobot(m *model.CloudIssuanceUserRobot) (id int, err error) {
	_, err = cloudIssuanceUserRobotDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	id = m.Id
	//顺便新增一份数据到总后台的 `cloud_issuance_robot_records` 表
	db.CloudIssuanceRobotRecordsInsert(&model2.CloudIssuanceRobotRecords{
		MasterId: cloudIssuanceUserRobotDb.MasterId,
		RobotId:  strconv.Itoa(m.RobotId),
		Uid:      cloudIssuanceUserRobotDb.Uid,
	})
	return id, nil
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) SaveCloudIssuanceUserRobot(id interface{}, m *model.CloudIssuanceUserRobot, forceColums ...string) (affected int64, err error) {
	if forceColums != nil {
		affected, err = cloudIssuanceUserRobotDb.Db.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = cloudIssuanceUserRobotDb.Db.Where("id=?", id).Update(m)
	}
	return
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) FindOnLineCloudIssuanceUserRobot() (m []*model.CloudIssuanceUserRobot, err error) {
	err = cloudIssuanceUserRobotDb.Db.Where("login_status =? AND is_enabled =0", enum.AlreadyLogin).OrderBy("id ASC").Find(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	return m, nil
}

func (cloudIssuanceUserRobotDb *CloudIssuanceUserRobotDb) GetCloudIssuanceUsingFreeRobot() (m []*model.CloudIssuanceUserRobot, err error) {
	err = cloudIssuanceUserRobotDb.Db.Where("uid != -1").
		And("robot_kind =?", enum.ShareRobot).
		And("is_enabled = 0").OrderBy("id ASC").Find(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	return m, nil
}
