package db

import (
	commDb "applet/app/db"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"time"
	"xorm.io/xorm"
)

type CloudIssuanceUserRobotBindFollowDb struct {
	Db  *xorm.Engine `json:"model"`
	Uid int          `json:"uid"`
}
type CloudIssuanceUserRobotBindFollowList struct {
	Id              int                                            `json:"id" xorm:"pk autoincr INT(11)"`
	Uid             int                                            `json:"uid" xorm:"not null default 0 comment('用户id') INT(11)"`
	Name            string                                         `json:"name" xorm:"comment('群名称')"`
	RobotId         int                                            `json:"robot_id" xorm:"comment('机器人id')"`
	ChatRoomId      string                                         `json:"chat_room_id" xorm:"comment('发送微信好友/群id')"`
	ChatRoomOwner   string                                         `json:"chat_room_owner" xorm:"comment('群所有者')"`
	MemberCount     int                                            `json:"member_count" xorm:"comment('群成员数')"`
	BigHeadImgUrl   string                                         `json:"big_head_img_url" xorm:"comment('大的群头像')"`
	SmallHeadImgUrl string                                         `json:"small_head_img_url" xorm:"comment('小的群头像')"`
	State           int                                            `json:"state" xorm:"not null default 1 comment('状态(1正常 2暂停)') TINYINT(1)"`
	CreateAt        time.Time                                      `json:"create_at" xorm:"not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	UpdateAt        time.Time                                      `json:"update_at" xorm:"not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	FollowUserList  []*model.CloudIssuanceUserRobotBindFollowUser  `json:"follow_user_list"`
	FollowGroupList []*model.CloudIssuanceUserRobotBindFollowGroup `json:"follow_group_list"`
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) Set(c *gin.Context, uid int) { // set方法
	cloudIssuanceUserRobotBindFollowDb.Uid = uid
	cloudIssuanceUserRobotBindFollowDb.Db = commDb.DBs[c.GetString("mid")]
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) GetCloudIssuanceUserRobotBindFollow(id int) (m *model.CloudIssuanceUserRobotBindFollow, err error) {
	m = new(model.CloudIssuanceUserRobotBindFollow)
	has, err := cloudIssuanceUserRobotBindFollowDb.Db.Where("id =?", id).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) GetCloudIssuanceUserRobotBindFollowByChatRoomId(chatRoomId string, robotId int) (m *model.CloudIssuanceUserRobotBindFollow, err error) {
	m = new(model.CloudIssuanceUserRobotBindFollow)
	has, err := cloudIssuanceUserRobotBindFollowDb.Db.Where("chat_room_id =? AND robot_id = ? AND state = 1", chatRoomId, robotId).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) InsertCloudIssuanceUserRobotBindFollow(m *model.CloudIssuanceUserRobotBindFollow) (id int, err error) {
	_, err = cloudIssuanceUserRobotBindFollowDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	id = m.Id
	return id, nil
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) SaveCloudIssuanceUserRobotBindFollow(id interface{}, m *model.CloudIssuanceUserRobotBindFollow, forceColums ...string) (affected int64, err error) {
	if forceColums != nil {
		affected, err = cloudIssuanceUserRobotBindFollowDb.Db.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = cloudIssuanceUserRobotBindFollowDb.Db.Where("id=?", id).Update(m)
	}
	return
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) DeleteCloudIssuanceUserRobotBindFollow(id interface{}) (affected int64, err error) {
	_, err = cloudIssuanceUserRobotBindFollowDb.Db.Where("id=?", id).Delete(model.CloudIssuanceUserRobotBindFollow{})
	if err != nil {
		return 0, err
	}
	return 1, nil
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) CountCloudIssuanceUserRobotBindFollow() (total int64, err error) {
	var m []*model.CloudIssuanceUserRobotBindFollow
	total, err = cloudIssuanceUserRobotBindFollowDb.Db.Where("uid=?", cloudIssuanceUserRobotBindFollowDb.Uid).OrderBy("id DESC").FindAndCount(&m)
	return
}

func (cloudIssuanceUserRobotBindFollowDb *CloudIssuanceUserRobotBindFollowDb) FindCloudIssuanceUserRobotBindFollow() (list []*CloudIssuanceUserRobotBindFollowList, err error) {
	var m []*model.CloudIssuanceUserRobotBindFollow
	err = cloudIssuanceUserRobotBindFollowDb.Db.Where("uid=?", cloudIssuanceUserRobotBindFollowDb.Uid).OrderBy("id DESC").Find(&m)
	if err != nil {
		return
	}

	for _, v := range m {
		var tmp CloudIssuanceUserRobotBindFollowList
		copier.Copy(&tmp, v)

		cloudIssuanceUserRobotBindFollowGroupDb := CloudIssuanceUserRobotBindFollowGroupDb{Db: cloudIssuanceUserRobotBindFollowDb.Db}
		groups, err := cloudIssuanceUserRobotBindFollowGroupDb.FindCloudIssuanceUserRobotBindFollowGroup(map[string]interface{}{
			"key":   "bind_id",
			"value": v.Id,
		})
		if err != nil {
			return list, err
		}
		tmp.FollowGroupList = groups

		cloudIssuanceUserRobotBindFollowUserDb := CloudIssuanceUserRobotBindFollowUserDb{Db: cloudIssuanceUserRobotBindFollowDb.Db}
		users, err := cloudIssuanceUserRobotBindFollowUserDb.FindCloudIssuanceUserRobotBindFollowUser(map[string]interface{}{
			"key":   "bind_id",
			"value": v.Id,
		})
		if err != nil {
			return list, err
		}
		tmp.FollowGroupList = groups
		tmp.FollowUserList = users

		list = append(list, &tmp)
	}
	return
}
