package db

import (
	commDb "applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	"errors"
	"fmt"
	"github.com/gin-gonic/gin"
	"reflect"
	"xorm.io/xorm"
)

type CloudIssuanceUserRobotBindFollowGroupDb struct {
	Db *xorm.Engine `json:"model"`
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) Set(c *gin.Context) { // set方法
	cloudIssuanceUserRobotBindFollowGroupDb.Db = commDb.DBs[c.GetString("mid")]
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) GetCloudIssuanceUserRobotBindFollowGroup(id int) (m *model.CloudIssuanceUserRobotBindFollowGroup, err error) {
	m = new(model.CloudIssuanceUserRobotBindFollowGroup)
	has, err := cloudIssuanceUserRobotBindFollowGroupDb.Db.Where("id =?", id).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) GetCloudIssuanceUserRobotBindFollowGroupByBindId(bandId int) (m []*model.CloudIssuanceUserRobotBindFollowGroup, err error) {
	err = cloudIssuanceUserRobotBindFollowGroupDb.Db.Where("bind_id =?", bandId).Find(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	return m, nil
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) InsertCloudIssuanceUserRobotBindFollowGroup(m *model.CloudIssuanceUserRobotBindFollowGroup) (id int, err error) {
	_, err = cloudIssuanceUserRobotBindFollowGroupDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	id = m.Id
	return id, nil
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) BatchAddCloudIssuanceUserRobotBindFollowGroups(data []*model.CloudIssuanceUserRobotBindFollowGroup) (int64, error) {
	affected, err := cloudIssuanceUserRobotBindFollowGroupDb.Db.Insert(data)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) SaveCloudIssuanceUserRobotBindFollowGroup(id interface{}, m *model.CloudIssuanceUserRobotBindFollowGroup, forceColums ...string) (affected int64, err error) {
	if forceColums != nil {
		affected, err = cloudIssuanceUserRobotBindFollowGroupDb.Db.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = cloudIssuanceUserRobotBindFollowGroupDb.Db.Where("id=?", id).Update(m)
	}
	return
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) DeleteCloudIssuanceUserRobotBindFollowGroup(bindId interface{}) (affected int64, err error) {
	_, err = cloudIssuanceUserRobotBindFollowGroupDb.Db.Where("bind_id=?", bindId).Delete(model.CloudIssuanceUserRobotBindFollowGroup{})
	if err != nil {
		return 0, err
	}
	return 1, nil
}

func (cloudIssuanceUserRobotBindFollowGroupDb *CloudIssuanceUserRobotBindFollowGroupDb) FindCloudIssuanceUserRobotBindFollowGroup(params map[string]interface{}) ([]*model.CloudIssuanceUserRobotBindFollowGroup, error) {
	var m []*model.CloudIssuanceUserRobotBindFollowGroup
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := cloudIssuanceUserRobotBindFollowGroupDb.Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := cloudIssuanceUserRobotBindFollowGroupDb.Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := cloudIssuanceUserRobotBindFollowGroupDb.Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return m, nil
		}

	}
}
