package db

import (
	commDb "applet/app/db"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/model"
	"applet/cloud_issuance/md"
	"github.com/gin-gonic/gin"
	"xorm.io/xorm"
)

type CloudIssuanceUserRobotBindGroupDb struct {
	Db  *xorm.Engine `json:"model"`
	Uid int          `json:"uid"`
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) Set(c *gin.Context, uid int) { // set方法
	cloudIssuanceUserRobotBindGroupDb.Uid = uid
	cloudIssuanceUserRobotBindGroupDb.Db = commDb.DBs[c.GetString("mid")]
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) GetCloudIssuanceUserRobotBindGroup(id int) (m *model.CloudIssuanceUserRobotBindGroup, err error) {
	m = new(model.CloudIssuanceUserRobotBindGroup)
	has, err := cloudIssuanceUserRobotBindGroupDb.Db.Where("id =?", id).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) GetCloudIssuanceUserRobotBindGroupByChatRoomId(chatRoomId string) (m *model.CloudIssuanceUserRobotBindGroup, err error) {
	m = new(model.CloudIssuanceUserRobotBindGroup)
	has, err := cloudIssuanceUserRobotBindGroupDb.Db.Where("chat_room_id =?", chatRoomId).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) InsertCloudIssuanceUserRobotBindGroup(m *model.CloudIssuanceUserRobotBindGroup) (id int, err error) {
	_, err = cloudIssuanceUserRobotBindGroupDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	id = m.Id
	return id, nil
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) SaveCloudIssuanceUserRobotBindGroup(id interface{}, m *model.CloudIssuanceUserRobotBindGroup, forceColums ...string) (affected int64, err error) {
	if forceColums != nil {
		affected, err = cloudIssuanceUserRobotBindGroupDb.Db.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = cloudIssuanceUserRobotBindGroupDb.Db.Where("id=?", id).Update(m)
	}
	return
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) DeleteCloudIssuanceUserRobotBindGroup(id interface{}) (affected int64, err error) {
	_, err = cloudIssuanceUserRobotBindGroupDb.Db.Where("id=?", id).Delete(model.CloudIssuanceUserRobotBindGroup{})
	if err != nil {
		return 0, err
	}
	return 1, nil
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) FindCloudIssuanceUserRobotBindGroup() (resp []*md.FindCloudIssuanceUserRobotBindGroupResponse, total int64, err error) {
	total, err = cloudIssuanceUserRobotBindGroupDb.Db.Table("cloud_issuance_user_robot_bind_group").
		Join("LEFT", "cloud_issuance_goods_category", "cloud_issuance_user_robot_bind_group.goods_category_id = cloud_issuance_goods_category.id").
		Where("cloud_issuance_user_robot_bind_group.uid = ?", cloudIssuanceUserRobotBindGroupDb.Uid).
		OrderBy("cloud_issuance_user_robot_bind_group.id DESC").
		Cols("cloud_issuance_user_robot_bind_group.*, cloud_issuance_goods_category.name as goods_category_name").
		FindAndCount(&resp)
	return
}

func (cloudIssuanceUserRobotBindGroupDb *CloudIssuanceUserRobotBindGroupDb) FindCloudIssuanceUserRobotBindGroupByCategory(categoryId int) (m []*model.CloudIssuanceUserRobotBindGroup, err error) {
	err = cloudIssuanceUserRobotBindGroupDb.Db.Where("uid =? AND goods_category_id =? AND state =1", cloudIssuanceUserRobotBindGroupDb.Uid, categoryId).OrderBy("id ASC").Find(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	return m, nil
}
