package db

import (
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/cloud_issuance/db/official/model"
	"errors"
	"fmt"
	"reflect"
)

// BatchSelectCloudIssuanceMaster 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `CloudIssuanceMasterFindByParams` 方法
func BatchSelectCloudIssuanceMaster(params map[string]interface{}) (*[]model.CloudIssuanceMaster, error) {
	var CloudIssuanceMasterData []model.CloudIssuanceMaster
	if err := db.Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&CloudIssuanceMasterData); err != nil {
		return nil, logx.Warn(err)
	}
	return &CloudIssuanceMasterData, nil
}

// CloudIssuanceMasterInsert 插入单条数据
func CloudIssuanceMasterInsert(CloudIssuanceMaster *model.CloudIssuanceMaster) (int, error) {
	_, err := db.Db.InsertOne(CloudIssuanceMaster)
	if err != nil {
		return 0, err
	}
	return CloudIssuanceMaster.Id, nil
}

// BatchAddCloudIssuanceMaster 批量新增数据
func BatchAddCloudIssuanceMaster(CloudIssuanceMasterData []*model.CloudIssuanceMaster) (int64, error) {
	affected, err := db.Db.Insert(CloudIssuanceMasterData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetCloudIssuanceMasterCount() int {
	var CloudIssuanceMaster model.CloudIssuanceMaster
	session := db.Db.Where("")
	count, err := session.Count(&CloudIssuanceMaster)
	if err != nil {
		return 0
	}
	return int(count)
}

// CloudIssuanceMasterDelete 删除记录
func CloudIssuanceMasterDelete(id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return db.Db.In("id", id).Delete(model.CloudIssuanceMaster{})
	} else {
		return db.Db.Where("id = ?", id).Delete(model.CloudIssuanceMaster{})
	}
}

// CloudIssuanceMasterUpdate 更新记录
func CloudIssuanceMasterUpdate(id interface{}, CloudIssuanceMaster *model.CloudIssuanceMaster, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = db.Db.Where("id=?", id).Cols(forceColums...).Update(CloudIssuanceMaster)
	} else {
		affected, err = db.Db.Where("id=?", id).Update(CloudIssuanceMaster)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// CloudIssuanceMasterGetOneByParams 通过传入的参数查询数据(单条)
func CloudIssuanceMasterGetOneByParams(params map[string]interface{}) (*model.CloudIssuanceMaster, error) {
	var m model.CloudIssuanceMaster
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := db.Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// CloudIssuanceMasterFindByParams 通过传入的参数查询数据(多条)
func CloudIssuanceMasterFindByParams(params map[string]interface{}) (*[]model.CloudIssuanceMaster, error) {
	var m []model.CloudIssuanceMaster
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := db.Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := db.Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := db.Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}
