package svc

import (
	"applet/app/cfg"
	"applet/app/utils"
	"applet/cloud_issuance/enum"
	"applet/cloud_issuance/md"
	"applet/cloud_issuance/tool"
	"encoding/json"
	"fmt"
	"github.com/gin-gonic/gin"
	"strconv"
	"strings"
)

type CloudIssuanceService struct {
	Context           *gin.Context `json:"context"`
	RequestMethodName string       `json:"request_method_name"`
	RequestClientIp   string       `json:"request_client_ip"`
}

func (svcCloudIssuance *CloudIssuanceService) Set(c *gin.Context, methodName, clientIp string) { // set方法
	svcCloudIssuance.Context = c
	svcCloudIssuance.RequestMethodName = methodName
	svcCloudIssuance.RequestClientIp = clientIp
}

func (svcCloudIssuance *CloudIssuanceService) RobotCreate(args md.RobotCreateRequest) (err error, result md.RobotCreateResponse) { // 添加机器人
	if !cfg.Prd {
		//TODO::测试环境不用买
		return
	}
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"month":       strconv.Itoa(args.Month),
		"robot_type":  args.RobotType,
		"wechatrobot": args.WechatRobot,
		"remark":      args.Remark,
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}

	return
}

func (svcCloudIssuance *CloudIssuanceService) QrcodeMacLogin(args md.RobotQrcodeMacLoginRequest) (err error, result md.RobotQrcodeMacLoginResponse) { // 获取登录二维码
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotAsyncMacLogin(args md.RobotAsyncMacLoginRequest) (err error, result md.RobotAsyncMacLoginResponse) { // 同步登录状态
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"wId":      args.WId,
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotForceOffline(args md.RobotForceOfflineRequest) (err error) { // 下线机器人
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
	})
	_, err = svcCloudIssuance.SendPost(url, args)
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotSecondLogin(args md.RobotSecondLoginRequest) (err error, result md.RobotSecondLoginResponse) { // 二次登录
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotChange(args md.RobotChangeRequest) (err error, result md.RobotChangeResponse) { // 机器人修改/续费
	if !cfg.Prd {
		//TODO::测试环境不用买
		return
	}
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"month":    strconv.Itoa(args.Month),
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotReset(args md.RobotResetRequest) (err error) { // 重置机器人
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
	})
	_, err = svcCloudIssuance.SendPost(url, args)
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotRoomList(args md.RobotRoomListRequest) (err error, result []md.RobotRoomDetailResponse) { // 群组列表
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	var list = md.RobotRoomListResponse{}
	err = json.Unmarshal(resByte, &list)
	if err != nil {
		return
	}

	//TODO::每次最多5个 调用"群组详情"接口
	i := 0
	var tmpList = md.RobotRoomListResponse{}
	var tmpResult []md.RobotRoomDetailResponse
	for i < len(list) {
		tmpList = append(tmpList, list[i])
		i++
		if len(tmpList) == 1 {
			roomId := strings.Join(tmpList, ",")
			requestUrl := tool.HttpBuild(enum.RobotRoomDetailMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
				"robot_id": strconv.Itoa(args.RobotId),
				"room_id":  roomId,
			})
			data, err = svcCloudIssuance.SendPost(requestUrl, md.RobotRoomDetailRequest{
				RobotId: args.RobotId,
				RoomId:  roomId,
			})
			resByte, err = json.Marshal(data.Data)
			if err != nil {
				return
			}
			err = json.Unmarshal(resByte, &tmpResult)
			if err != nil {
				return
			}
			result = append(result, tmpResult...)
			tmpList = md.RobotRoomListResponse{}
		}
	}

	if len(tmpList) > 0 {
		roomId := strings.Join(list, ",")
		requestUrl := tool.HttpBuild(enum.RobotRoomDetailMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
			"robot_id": strconv.Itoa(args.RobotId),
			"room_id":  roomId,
		})
		data, err = svcCloudIssuance.SendPost(requestUrl, md.RobotRoomDetailRequest{
			RobotId: args.RobotId,
			RoomId:  roomId,
		})
		resByte, err = json.Marshal(data.Data)
		if err != nil {
			return
		}
		err = json.Unmarshal(resByte, &tmpResult)
		if err != nil {
			return
		}
		result = append(result, tmpResult...)
		tmpList = md.RobotRoomListResponse{}
	}

	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotMacGetChatRoomMember(args md.RobotMacGetChatRoomMemberRequest) (err error, result []md.RobotMacGetChatRoomMemberResponseV1) { // 获取群成员
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"room_id":  args.RoomId,
	})
	data, err := svcCloudIssuance.SendPost(url, args)
	if err != nil {
		return
	}
	resByte, err := json.Marshal(data.Data)
	if err != nil {
		return
	}
	err = json.Unmarshal(resByte, &result)
	if err != nil {
		return
	}
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotMacSendCard(args md.RobotMacSendCardRequest) (err error) { // 发送链接消息
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id":    strconv.Itoa(args.RobotId),
		"wx_id":       args.WxId,
		"title":       args.Title,
		"url":         args.Url,
		"description": args.Description,
		"thumbUrl":    args.ThumbUrl,
	})
	_, err = svcCloudIssuance.SendPost(url, args)
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotMacSendUri(args md.RobotMacSendUriRequest) (err error) { // 发送uri图片消息
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"toWxId":   args.WxId,
		"pic_url":  args.PicUrl,
	})
	_, err = svcCloudIssuance.SendPost(url, args)
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotMacSendText(args md.RobotMacSendTextRequest) (err error) { // 发送文本消息
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"toWxId":   args.WxId,
		"content":  args.Content,
	})
	resp, err := svcCloudIssuance.SendPost(url, args)
	if args.RobotId == 487259 {
		utils.FilePutContents("RobotMacSendText_487259", utils.SerializeStr(resp))
	}
	return
}

func (svcCloudIssuance *CloudIssuanceService) RobotMacSendRecvImage(args md.RobotMacSendRecvImageRequest) (err error) { // 发送文本消息
	url := tool.HttpBuild(svcCloudIssuance.RequestMethodName, svcCloudIssuance.RequestClientIp, map[string]string{
		"robot_id": strconv.Itoa(args.RobotId),
		"wx_id":    args.WxId,
		"content":  args.Content,
	})
	_, err = svcCloudIssuance.SendPost(url, args)
	return
}

func (svcCloudIssuance *CloudIssuanceService) SendPost(url string, args interface{}) (data md.CurlResponse, err error) {
	data, err = tool.SendPost(url, args)
	if data.Status == "1123" && data.Msg == "当前机器人不在线" && svcCloudIssuance.Context != nil {
		fmt.Println(">>>>>>>>>>>>>>>>>>当前机器人不在线<<<<<<<<<<<<<<<<<<<<<<", data.Data)
		//cloudIssuanceUserRobotDb := db.CloudIssuanceUserRobotDb{}
		//user := svcComm.GetUser(svcCloudIssuance.Context)
		//cloudIssuanceUserRobotDb.Set(svcCloudIssuance.Context, user.Info.Uid)
		//robot, err := cloudIssuanceUserRobotDb.GetCloudIssuanceUserRobot()
		//if err != nil {
		//	panic(err)
		//}
		//robot.LoginStatus = enum.NotLogin
		//cloudIssuanceUserRobotDb.SaveCloudIssuanceUserRobot(robot.Id, robot, "login_status")
	}
	return
}
