package consume

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/streadway/amqp"
	"strings"
	"time"
)

func CanalUserVirtualCoinFlowConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>CanalUserVirtualCoinFlowConsume>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalUserVirtualCoinFlowConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleCanalUserVirtualCoinFlow(res.Body)
			if err != nil {
				fmt.Println("handleCanalUserVirtualCoinFlow_ERR:::::", err.Error())
			}
			//_ = res.Reject(false)
			err = res.Ack(true)
			fmt.Println("err ::: ", err)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleCanalUserVirtualCoinFlow(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalUserVirtualCoinFlowOrderMessage[md.CanalUserVirtualCoinFlowOrder]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}

	masterId := strings.Split(canalMsg.Database, "_")[1]
	if masterId != "32053480" {
		return nil
	}
	engine := db.DBs[masterId]
	now := time.Now()

	//2、查找 one_circles_green_energy_basic_setting 基础设置
	userPublicPlatoonDoubleNetworkSetting, err := db.UserPublicPlatoonDoubleNetworkSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return err
	}
	if userPublicPlatoonDoubleNetworkSetting == nil {
		return errors.New("公排双网未开启")
	}
	if canalMsg.Type == md.CanalMsgInsertSqlType {
		if canalMsg.Data[0].CoinId == utils.IntToStr(userPublicPlatoonDoubleNetworkSetting.CoinId) {
			//3、查找 user_public_platoon_double_network_user_coin_record
			userPublicPlatoonDoubleNetworkUserCoinRecord, err1 := db.UserPublicPlatoonDoubleNetworkUserCoinRecordGetOneByParams(engine, map[string]interface{}{
				"key":   "uid",
				"value": canalMsg.Data[0].Uid,
			})
			if err1 != nil {
				return err1
			}
			if userPublicPlatoonDoubleNetworkUserCoinRecord == nil {
				userProfile, err2 := db.UserProfileFindByIDSess(engine.NewSession(), canalMsg.Data[0].Uid)
				if userProfile == nil {
					return errors.New("用户不存在")
				}
				if err2 != nil {
					return err2
				}
				//新增记录
				_, err3 := db.UserPublicPlatoonDoubleNetworkUserCoinRecordInsert(engine, &model.UserPublicPlatoonDoubleNetworkUserCoinRecord{
					Uid:          utils.StrToInt(canalMsg.Data[0].Uid),
					LastAmount:   canalMsg.Data[0].AfterAmout,
					Amount:       canalMsg.Data[0].AfterAmout,
					RecommendUid: userProfile.ParentUid,
					CoinId:       utils.StrToInt(canalMsg.Data[0].CoinId),
					CreateAt:     now.Format("2006-01-02 15:04:05"),
					UpdateAt:     now.Format("2006-01-02 15:04:05"),
				})
				if err3 != nil {
					return err3
				}
			} else {
				//更新记录
				afterAmount, _ := decimal.NewFromString(canalMsg.Data[0].AfterAmout)
				amount, _ := decimal.NewFromString(userPublicPlatoonDoubleNetworkUserCoinRecord.Amount)
				lastAmount, _ := decimal.NewFromString(userPublicPlatoonDoubleNetworkUserCoinRecord.LastAmount)
				if canalMsg.Data[0].Direction == "1" || canalMsg.Data[0].Direction == "2" {
					//收入 && 支出
					userPublicPlatoonDoubleNetworkUserCoinRecord.Amount = amount.Add(afterAmount.Sub(lastAmount)).String()
				}
				userPublicPlatoonDoubleNetworkUserCoinRecord.LastAmount = canalMsg.Data[0].AfterAmout
				_, err2 := db.UserPublicPlatoonDoubleNetworkUserCoinRecordUpdate(engine, userPublicPlatoonDoubleNetworkUserCoinRecord.Id, userPublicPlatoonDoubleNetworkUserCoinRecord, "amount", "last_amount")
				if err2 != nil {
					return err2
				}
			}
		}
	}

	return nil
}
