package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	db2 "applet/app/db/gim"
	model2 "applet/app/db/gim/model"
	"applet/app/db/model"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	utils "applet/app/utils/rpc"
	"applet/consume/md"
	"applet/pkg/pb"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strconv"
	"time"
)

func DouShenUserRegisterConsumeForOfficial(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>DouShenUserRegisterConsumeForOfficial>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleDouShenUserRegisterConsumeForOfficial(res.Body)
			if err != nil {
				fmt.Println("!!!!!!!err!!!!!!!", err)
				_ = res.Reject(false)

				utils2.FilePutContents("douShenUserRegisterConsumeForOfficial_err", "[err]:"+err.Error())
				if err.Error() == "用户暂未注册im系统" {
					//TODO::重新推回队列末尾，避免造成队列堵塞
					var msg *md.DouShenUserRegisterMessageStructForOfficial
					json.Unmarshal(res.Body, &msg)
					ch.Publish(queue.ExchangeName, msg, queue.RoutKey)
				}
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleDouShenUserRegisterConsumeForOfficial(msgData []byte) error {
	time.Sleep(time.Duration(5) * time.Second)
	//1、解析mq中queue的数据结构体
	var msg *md.DouShenUserRegisterMessageStructForOfficial
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}
	//2、查找用户对应im系统中的数据
	gimUser, err := db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
		"key":   "phone_number",
		"value": msg.Phone,
	})
	if err != nil {
		return err
	}
	if gimUser == nil {
		if db.DBs[msg.MasterId] == nil {
			return nil
		}
		fmt.Println("===========================", msg.Phone)
		user, err := db.UserFindByMobile(db.DBs[msg.MasterId], strconv.FormatInt(msg.Phone, 10))
		if err != nil {
			return err
		}
		if user == nil {
			return nil
		}
		userProfile, err := db.UserProfileFindByID(db.DBs[msg.MasterId], user.Uid)
		if err != nil {
			return err
		}

		gimUserId, err := db2.UserInsert(db.ImDb, &model2.User{
			PhoneNumber:        strconv.FormatInt(msg.Phone, 10),
			Nickname:           user.Nickname,
			AvatarUrl:          userProfile.AvatarUrl,
			Sex:                userProfile.Gender,
			CreateTime:         time.Now().Format("2006-01-02 15:04:05"),
			UpdateTime:         time.Now().Format("2006-01-02 15:04:05"),
			MasterId:           utils2.StrToInt64(msg.MasterId),
			IsAutoAddedFriends: 0,
		})
		if err != nil {
			return errors.New("用户暂未注册im系统")
		}
		gimUser, err = db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
			"key":   "id",
			"value": gimUserId,
		})
		if gimUser == nil {
			return errors.New("用户暂未注册im系统")
		}
	}

	//2、查找是否有群
	//officialGroup, err := db.DouShenImGroupGetOneByParamsForOfficial(db.Db, 1)
	var officialGroup model.DouShenImGroup
	officialGroups, err := db.DouShenImGroupFindByParams(db.Db, map[string]interface{}{
		"key":   "kind",
		"value": 1,
	})
	if err != nil {
		return err
	}
	for _, group := range *officialGroups {
		if officialGroup.GroupId != 0 {
			continue
		}
		//统计当前群有多少人
		userGroups, _ := db2.GroupUserFindByParams(db.ImDb, map[string]interface{}{
			"key":   "group_id",
			"value": group.GroupId,
		})
		if len(*userGroups) < 500 {
			officialGroup = group
		}
	}

	if officialGroup.GroupId == 0 {
		groupName := "官方【" + utils2.AnyToString(len(*officialGroups)+1) + "】群"
		//3、创建群
		resp, err := utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).CreateGroup(utils.GetCtx("", strconv.FormatInt(gimUser.Id, 10), "0", msg.MasterId), &pb.CreateGroupReq{
			Name:      groupName,
			AvatarUrl: "",
			//Introduction: "官方群",
			Introduction: "",
			Extra:        "",
			MemberIds:    []int64{},
		})
		if err != nil {
			return err
		}

		//插入 dou_shen_im_group 记录
		now := time.Now()
		_, err = db.DouShenImGroupInsert(db.Db, &model.DouShenImGroup{
			Kind:       1,
			Uid:        int(msg.Uid),
			GroupId:    int(resp.GroupId),
			IsFull:     0,
			Name:       groupName,
			CreateTime: now,
			UpdateTime: now,
		})
		if err != nil {
			return err
		}
	} else {
		userGroup, err2 := db2.GroupUserGetOneByParams(db.ImDb, 1, map[string]interface{}{
			"key":   "group_id",
			"value": officialGroup.GroupId,
		})
		if err2 != nil {
			return err2
		}
		if userGroup == nil {
			return errors.New("当前官方群暂未设置群主，请联系管理员！！！")
		}

		//加入群
		_, err = utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).AddGroupMembers(
			utils.GetCtx("", strconv.FormatInt(userGroup.UserId, 10), "", msg.MasterId),
			&pb.AddGroupMembersReq{
				GroupId: int64(officialGroup.GroupId),
				UserIds: []int64{gimUser.Id},
			})
	}
	return nil
}
