package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	db2 "applet/app/db/gim"
	"applet/app/db/gim/model"
	model2 "applet/app/db/model"
	"applet/app/svc"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	utils "applet/app/utils/rpc"
	"applet/consume/md"
	"applet/pkg/pb"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strconv"
	"time"
)

func DouShenUserRegisterConsumeForOperationCenter(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>DouShenUserRegisterConsumeForOperationCenter>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleDouShenUserRegisterConsumeForOperationCenter(res.Body)
			if err != nil {
				fmt.Println("!!!!!!!err!!!!!!!", err)
				_ = res.Reject(false)

				utils2.FilePutContents("douShenUserRegisterConsumeForOperationCenter_err", "[err]:"+err.Error())
				if err.Error() == "用户暂未注册im系统" {
					//TODO::重新推回队列末尾，避免造成队列堵塞
					var msg *md.DouShenUserRegisterMessageStructForOperationCenter
					json.Unmarshal(res.Body, &msg)
					ch.Publish(queue.ExchangeName, msg, queue.RoutKey)
				}
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleDouShenUserRegisterConsumeForOperationCenter(msgData []byte) error {
	time.Sleep(time.Duration(30) * time.Second)
	//1、解析mq中queue的数据结构体
	var msg *md.DouShenUserRegisterMessageStructForOperationCenter
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}
	//2、查找用户对应im系统中的数据
	gimUser, err := db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
		"key":   "phone_number",
		"value": msg.Phone,
	})
	if err != nil {
		return err
	}

	if gimUser == nil {
		if db.DBs[msg.MasterId] == nil {
			return nil
		}
		user, err := db.UserFindByMobile(db.DBs[msg.MasterId], strconv.FormatInt(msg.Phone, 10))
		if err != nil {
			return err
		}
		if user == nil {
			return nil
		}
		userProfile, err := db.UserProfileFindByID(db.DBs[msg.MasterId], user.Uid)
		if err != nil {
			return err
		}
		if userProfile == nil {
			return nil
		}
		gimUserId, err := db2.UserInsert(db.ImDb, &model.User{
			PhoneNumber:        strconv.FormatInt(msg.Phone, 10),
			Nickname:           user.Nickname,
			AvatarUrl:          userProfile.AvatarUrl,
			Sex:                userProfile.Gender,
			CreateTime:         time.Now().Format("2006-01-02 15:04:05"),
			UpdateTime:         time.Now().Format("2006-01-02 15:04:05"),
			MasterId:           utils2.StrToInt64(msg.MasterId),
			IsAutoAddedFriends: 0,
		})
		if err != nil {
			return errors.New("用户暂未注册im系统")
		}
		gimUser, err = db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
			"key":   "id",
			"value": gimUserId,
		})
		if gimUser == nil {
			return errors.New("用户暂未注册im系统")
		}
	}

	//2、查找是否有群
	//OperationGroup, err := db.DouShenImGroupGetOneByParamsForFans(db.Db, 2, msg.OperationCenterUid)
	var OperationGroup model2.DouShenImGroup
	operationGroups, err := db.DouShenImGroupFindByParamsByKind(db.Db, 2, map[string]interface{}{
		"key":   "uid",
		"value": msg.OperationCenterUid,
	})
	if err != nil {
		return err
	}
	for _, group := range *operationGroups {
		//统计当前群有多少人
		userGroups, err := db2.GroupUserFindByParams(db.ImDb, map[string]interface{}{
			"key":   "group_id",
			"value": group.GroupId,
		})
		if err != nil {
			return err
		}
		if len(*userGroups) < 500 {
			OperationGroup = group
		}
	}

	if OperationGroup.GroupId == 0 {
		//创建运营中心群
		operationCenterUser, err := db.UserFindByID(db.DBs[msg.MasterId], msg.OperationCenterUid)
		if err != nil {
			return err
		}
		operationCenterGimUser, err := db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
			"key":   "phone_number",
			"value": operationCenterUser.Phone,
		})
		if operationCenterGimUser == nil {
			return errors.New("运营中心用户暂未注册im系统")
		}
		var m model2.RegionalAgentUser
		_, err = db.DBs[msg.MasterId].Where("uid =?", msg.OperationCenterUid).Get(&m)
		if err != nil {
			panic(err)
		}
		regionalAgentRegion, err := db.RegionalAgentRegionGetOneByParams(db.DBs[msg.MasterId], map[string]interface{}{
			"key":   "id",
			"value": m.RegionId,
		})
		if err != nil {
			panic(err)
		}

		groupName := "抖省" + regionalAgentRegion.Name + "运营中心【" + utils2.AnyToString(len(*operationGroups)) + "】群"

		resp, err := utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).CreateGroup(utils.GetCtx("", utils2.Int64ToStr(operationCenterGimUser.Id), "", msg.MasterId), &pb.CreateGroupReq{
			Name:      groupName,
			AvatarUrl: "",
			//Introduction: gimUser.Nickname + "的粉丝群",
			Introduction: "",
			Extra:        "",
			MemberIds:    nil,
		})
		if err != nil {
			return err
		}
		//插入 dou_shen_im_group 记录
		now := time.Now()
		_, err = db.DouShenImGroupInsert(db.Db, &model2.DouShenImGroup{
			Kind:       2,
			Uid:        int(msg.OperationCenterUid),
			GroupId:    int(resp.GroupId),
			IsFull:     0,
			Name:       groupName,
			CreateTime: now,
			UpdateTime: now,
		})
		if err != nil {
			return err
		}
		tmpOperationGroup, err := db.DouShenImGroupGetOneByParams(db.Db, 2, map[string]interface{}{
			"group_id": resp.GroupId,
		})
		if err != nil {
			return err
		}
		OperationGroup = *tmpOperationGroup
	}
	userGroup, err2 := db2.GroupUserGetOneByParams(db.ImDb, 1, map[string]interface{}{
		"key":   "group_id",
		"value": OperationGroup.GroupId,
	})
	if err2 != nil {
		return err2
	}
	if userGroup == nil {
		return errors.New("当前运营中心群暂未设置群主，请联系管理员！！！")
	}

	//加入群
	_, err = utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).AddGroupMembers(utils.GetCtx("", strconv.FormatInt(userGroup.UserId, 10), "", msg.MasterId), &pb.AddGroupMembersReq{
		GroupId: int64(OperationGroup.GroupId),
		UserIds: []int64{gimUser.Id},
	})

	//发送专属红包
	gimSendSpeciallyRedPackageUser, err := db2.DbSysCfg.SysCfgGetOne("send_specially_red_package_user", msg.MasterId)
	if err != nil {
		return err
	}
	if gimSendSpeciallyRedPackageUser == nil {
		return errors.New("暂未设置专属红包发送用户")
	}
	sendSpeciallyRedPackageUser, err := db.SysCfgGetOne(db.DBs[msg.MasterId], "send_specially_red_package_user")
	if err != nil {
		return err
	}
	amount, err := getSendAmount(msg.MasterId)
	if err != nil {
		return err
	}
	args := md.SendRedPackageReq{
		UserId:                gimSendSpeciallyRedPackageUser.Val,
		DeviceId:              "",
		Token:                 "",
		Amount:                amount,
		RedPacketType:         5,
		RedPacketNums:         1,
		RedPacketContent:      "抖省赚钱，大吉大利",
		RedPacketSmallContent: gimUser.Nickname + "的专属红包",
		ReceiverType:          2,
		ReceiverId:            int64(OperationGroup.GroupId),
		SendTime:              utils2.UnixMilliTime(time.Now()),
		ToUserIds:             []int64{gimUser.Id},
		RedPackageCover:       "",
	}
	err, _ = svc.CurlSendRedPackage(args, msg.MasterId, sendSpeciallyRedPackageUser.Val)
	if err != nil {
		return err
	}
	return nil
}

func getSendAmount(masterId string) (string, error) {
	speciallyRedPackageAmountRadio, err := db.SysCfgGetOne(db.DBs[masterId], "specially_red_package_amount_radio")
	if err != nil {
		return "", err
	}
	var speciallyRedPackageAmountRadioData []*md.SpeciallyRedPackageAmountRadio
	err = json.Unmarshal([]byte(speciallyRedPackageAmountRadio.Val), &speciallyRedPackageAmountRadioData)
	if err != nil {
		return "", err
	}

	var tempSpeciallyRedPackageAmountRadioData []*md.SpeciallyRedPackageAmountRadio
	for _, v := range speciallyRedPackageAmountRadioData {
		for i := 0; i < utils2.StrToInt(v.Value); i++ {
			tempSpeciallyRedPackageAmountRadioData = append(tempSpeciallyRedPackageAmountRadioData, v)
		}
	}
	if len(tempSpeciallyRedPackageAmountRadioData) < 100 {
		return "", errors.New("红包金额比例设置有误！")
	}

	randInt := utils2.RandInt(99)
	min := tempSpeciallyRedPackageAmountRadioData[randInt].From
	max := tempSpeciallyRedPackageAmountRadioData[randInt].To
	amount := utils2.AnyToString(utils2.RandFloats(utils2.AnyToFloat64(min), utils2.AnyToFloat64(max)))
	return amount, nil
}
