package md

import "applet/app/utils"

type CanalOrder struct {
	OrdId                 string `json:"ord_id"`
	MainOrdId             string `json:"main_ord_id"`
	StoreOrdId            string `json:"store_ord_id"`
	GoodsId               string `json:"goods_id"`
	Uid                   string `json:"uid"`
	BuyerPhone            string `json:"buyer_phone"`
	CostPrice             string `json:"cost_price"`
	State                 string `json:"state"`
	PayTime               string `json:"pay_time"`
	PayChannel            string `json:"pay_channel"`
	ReceiverPhone         string `json:"receiver_phone"`
	ReceiverName          string `json:"receiver_name"`
	ReceiverAddressDetail string `json:"receiver_address_detail"`
	ProvinceId            string `json:"province_id"`
	CityId                string `json:"city_id"`
	CountyId              string `json:"county_id"`
	OrderType             string `json:"order_type"`
	EstimateCommission    string `json:"estimate_commission"`
	CreateTime            string `json:"create_time"`
	LogisticCompany       string `json:"logistic_company"`
}

type CanalOrderMessage[T any] struct {
	Data     []T      `json:"data"`
	Database string   `json:"database"`
	ES       int64    `json:"es"`
	ID       int64    `json:"id"`
	IsDdl    bool     `json:"isDdl"`
	Old      []T      `json:"old"`
	PkNames  []string `json:"pkNames"`
	Table    string   `json:"table"`
	TS       int64    `json:"ts"`
	Type     string   `json:"type"`
}

type CanalOrderState int32

const (
	CanalOrderStateForWaitPay CanalOrderState = iota
	CanalOrderStateForAlreadyPay
	CanalOrderStateForAlreadySendOut
	CanalOrderStateForAlreadyComplete
	CanalOrderStateForAfterSales
	CanalOrderStateForPartAfterSales
	CanalOrderStateForClose
)

func (state CanalOrderState) String() string {
	switch state {
	case CanalOrderStateForWaitPay:
		return "未支付"
	case CanalOrderStateForAlreadyPay:
		return "已支付"
	case CanalOrderStateForAlreadySendOut:
		return "已发货"
	case CanalOrderStateForAlreadyComplete:
		return "已完成"
	case CanalOrderStateForAfterSales:
		return "售后中"
	case CanalOrderStateForPartAfterSales:
		return "部分售后中"
	case CanalOrderStateForClose:
		return "关闭"
	default:
		return "未知:" + utils.AnyToString(state)
	}
}

type CanalOrderPayChannel int32

const (
	CanalOrderPayChannelForBalancePay CanalOrderPayChannel = iota
	CanalOrderPayChannelForAliPay
	CanalOrderPayChannelForWxPay
	CanalOrderPayChannelForMoICBCPay
	CanalOrderPayChannelForMoICBCPayAli
	CanalOrderPayChannelForMOICBCEPay
	CanalOrderPayChannelForMoICBCPayWx
	CanalOrderPayChannelForDaFengPay
	CanalOrderPayChannelForMoPassPay
	CanalOrderPayChannelForJoinPayAliPay
	CanalOrderPayChannelForJoinPayWxPay
	CanalOrderPayChannelForJoinPayBankCard
	CanalOrderPayChannelForAdaPayBankCard
	CanalOrderPayChannelForMoBocPay
	CanalOrderPayChannelForAdaPatAli
	CanalOrderPayChannelForAdaPatWx
	CanalOrderPayChannelForMoLusoPay
	CanalOrderPayChannelForFbPayAli
	CanalOrderPayChannelForFbPayWx
	CanalOrderPayChannelForFbPayFast
)

func (state CanalOrderPayChannel) String() string {
	switch state {
	case CanalOrderPayChannelForBalancePay:
		return "余额支付"
	case CanalOrderPayChannelForAliPay:
		return "支付宝支付"
	case CanalOrderPayChannelForWxPay:
		return "微信支付"
	case CanalOrderPayChannelForMoICBCPay:
		return "工銀支付"
	case CanalOrderPayChannelForMoICBCPayAli:
		return "澳門工銀支付-支付宝支付"
	case CanalOrderPayChannelForMOICBCEPay:
		return "E支付"
	case CanalOrderPayChannelForMoICBCPayWx:
		return "澳門工銀支付-微信支付"
	case CanalOrderPayChannelForDaFengPay:
		return "豐付寶"
	case CanalOrderPayChannelForMoPassPay:
		return "MPay"
	case CanalOrderPayChannelForJoinPayAliPay:
		return "汇聚支付宝支付"
	case CanalOrderPayChannelForJoinPayWxPay:
		return "汇聚微信支付"
	case CanalOrderPayChannelForJoinPayBankCard:
		return "银行卡快捷支付"
	case CanalOrderPayChannelForAdaPayBankCard:
		return "汇付快捷支付"
	case CanalOrderPayChannelForMoBocPay:
		return "智慧付"
	case CanalOrderPayChannelForAdaPatAli:
		return "汇付支付宝支付"
	case CanalOrderPayChannelForAdaPatWx:
		return "汇付微信支付"
	case CanalOrderPayChannelForMoLusoPay:
		return "澳門國際銀行支付"
	case CanalOrderPayChannelForFbPayAli:
		return "乐刷支付宝支付"
	case CanalOrderPayChannelForFbPayWx:
		return "乐刷微信支付"
	case CanalOrderPayChannelForFbPayFast:
		return "乐刷快捷支付"
	default:
		return "未知:" + utils.AnyToString(state)
	}
}
