package consume

import (
	"applet/app/db"
	"applet/app/db/offical"
	"applet/app/db/offical/model"
	"applet/app/e"
	"applet/app/svc"
	"applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_third_party_api.git/express"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"github.com/tidwall/gjson"
	"time"
)

func ZhiosExpressOrderFail(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(20)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleZhiosExpressOrderFail(res.Body)
			//_ = res.Reject(false)
			if err == nil {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleZhiosExpressOrderFail(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.ZhiosExpressOrderFails
	fmt.Println(string(msg))
	var tmpString string
	err := json.Unmarshal(msg, &tmpString)
	if err != nil {
		fmt.Println(err.Error())
		return err
	}
	fmt.Println(tmpString)
	err = json.Unmarshal([]byte(tmpString), &canalMsg)
	if err != nil {
		return err
	}
	mid := canalMsg.Mid
	eg := db.DBs[mid]
	if eg == nil {
		return nil
	}
	//判断订单是否订单失败
	ord := db.GetExpressWithOid(eg, canalMsg.Oid)
	officialOrd := offical.GetExpressWithOid(canalMsg.Oid)
	if ord.Status == "已退回" {
		return nil
	}
	base := CommBase(mid)
	if canalMsg.IsFail != "1" { //查下订单详情
		param := map[string]interface{}{
			"clientOrderNo": canalMsg.Oid,
		}
		order, _ := express.ShowOrder(base, param)
		if gjson.Get(order, "code").Int() == 500 || gjson.Get(order, "code").String() != "00" || gjson.Get(order, "data.status").String() == "CANCELED" {
			ord.Status = "订单失败"
		}
	}
	if ord.Status == "订单失败" || ord.Status == "已取消" {
		ord.Status = "已退回"
		ord.IsRefund = 1
		ord.RefundTime = time.Now()
		officialOrd.Status = "已退回"
		officialOrd.IsRefund = 1
		officialOrd.RefundTime = time.Now()
		eg.Where("oid=?", ord.Oid).Update(ord)
		db.Db.Where("oid=?", officialOrd.Oid).Update(officialOrd)
		svc.DealMoneyWithEg(eg, ord.Uid, ord.Money, 56, utils.StrToInt64(ord.Oid), 0, 0, "快递退款", "express", 0)
		if ord.AgentPay == 1 {
			agentDeduct(ord.Oid, base, ord.AgentPrice)
		}
		if ord.StationPay == 1 {
			stationDeduct(mid, ord.Oid, base, ord.StationPrice)
		}
	}
	return nil
}

//站长预存款扣除
func stationDeduct(dbName, oid string, base map[string]string, price string) (error, int) {
	eg := db.Db
	var user model.ExpressUserMoney
	get, err := eg.Where("uid=? and agent_uid=?", dbName, base["puid"]).Get(&user)
	if get == false || err != nil {
		return e.NewErr(400, "站点预存款不足!"), 0
	}
	user.Money = utils.Float64ToStr(utils.StrToFloat64(user.Money) + utils.StrToFloat64(price))
	_, err = eg.Where("id=?", user.Id).Update(&user)
	if err != nil {
		return e.NewErr(400, "站点预存款扣除失败!"), 0
	}
	var flow = &model.ExpressUserMoneyFlow{
		Uid:         user.Uid,
		Type:        0,
		Time:        time.Now(),
		Amount:      price,
		AfterAmount: user.Money,
		Oid:         oid,
		Title:       "快递退款",
	}
	_, err = eg.Insert(flow)
	if err != nil {
		return e.NewErr(400, "站点预存款扣除失败!"), 0
	}
	return nil, 1
}
func agentDeduct(oid string, base map[string]string, price string) (error, int) {
	eg := db.Db
	var user model.ExpressAgentMoney
	get, err := eg.Where("uid=?", base["puid"]).Get(&user)
	if get == false || err != nil {
		return e.NewErr(400, "平台预存款不足!"), 0
	}
	user.Money = utils.Float64ToStr(utils.StrToFloat64(user.Money) + utils.StrToFloat64(price))
	_, err = eg.Where("id=?", user.Id).Update(&user)
	if err != nil {
		return e.NewErr(400, "平台预存款扣除失败!"), 0
	}
	var flow = &model.ExpressAgentMoneyFlow{
		Uid:         user.Uid,
		Type:        0,
		Time:        time.Now(),
		Amount:      price,
		AfterAmount: user.Money,
		Oid:         oid,
		Title:       "快递退款",
	}
	_, err = eg.Insert(flow)
	if err != nil {
		return e.NewErr(400, "平台预存款扣除失败!"), 0
	}
	return nil, 1
}

func AppUserListPuid(mid string) string {
	appList := offical.GetUserAppList(mid)
	uid := "0"
	if appList != nil && appList.Puid > 0 {
		uid = utils.IntToStr(appList.Puid)
	}
	return uid
}

func CommBase(mid string) map[string]string {
	puid := AppUserListPuid(mid)
	key := puid + "_official_express_info"
	stringStr, err := cache.GetString(key)
	stringMap := make(map[string]string)
	json.Unmarshal([]byte(stringStr), &stringMap)
	if len(stringMap) == 0 || err != nil {
		expressType := offical.MasterListCfgGetOneData(puid, "express_type")
		expressUrl := offical.MasterListCfgGetOneData(puid, "express_url")
		expressClientId := offical.MasterListCfgGetOneData(puid, "express_client_id")
		expressKey := offical.MasterListCfgGetOneData(puid, "express_key")
		expressMinSendMoney := offical.MasterListCfgGetOneData(puid, "express_min_send_money")   //最低预存款
		expressFeeSendMoney := offical.MasterListCfgGetOneData(puid, "express_fee_send_money")   //充值预存款手续费
		expressFirstPayBili := offical.MasterListCfgGetOneData(puid, "express_first_pay_bili")   //首重加价
		expressSecondPayBili := offical.MasterListCfgGetOneData(puid, "express_second_pay_bili") //续重加价
		stringMap = map[string]string{
			"puid":                             puid,
			"express_type":                     expressType,
			"express_url":                      expressUrl,
			"express_key":                      expressKey,
			"express_client_id":                expressClientId,
			"express_min_send_money":           "",
			"express_fee_send_money":           "",
			"express_first_pay_bili":           "",
			"express_second_pay_bili":          "",
			"official_express_min_send_money":  expressMinSendMoney,
			"official_express_fee_send_money":  expressFeeSendMoney,
			"official_express_first_pay_bili":  expressFirstPayBili,
			"official_express_second_pay_bili": expressSecondPayBili,
		}
		//如果是官方 不是代理就没有代理的设置
		//如果是代理 就有代理的设置
		if utils.StrToInt(puid) > 0 {
			stringMap["express_min_send_money"] = expressMinSendMoney
			stringMap["express_fee_send_money"] = expressFeeSendMoney
			stringMap["express_first_pay_bili"] = expressFirstPayBili
			stringMap["express_second_pay_bili"] = expressSecondPayBili
			if stringMap["express_type"] == "1" { //代理自有渠道,智莺的加价不用了
				stringMap["official_express_min_send_money"] = ""
				stringMap["official_express_fee_send_money"] = ""
				stringMap["official_express_first_pay_bili"] = ""
				stringMap["official_express_second_pay_bili"] = ""
			}
		}
		cache.SetEx(key, utils.SerializeStr(stringMap), 300)

	}
	if utils.StrToInt(puid) > 0 && stringMap["express_type"] != "1" { //跟随官方
		key1 := "0_official_express_info"
		stringStr1, err1 := cache.GetString(key1)
		stringMap1 := make(map[string]string)
		json.Unmarshal([]byte(stringStr1), &stringMap1)
		if len(stringMap1) == 0 || err1 != nil { //如果是代理 但只有官方设置
			stringMap1 = make(map[string]string)

			stringMap1["express_url"] = offical.MasterListCfgGetOneData("0", "express_url")
			stringMap1["express_client_id"] = offical.MasterListCfgGetOneData("0", "express_client_id")
			stringMap1["express_key"] = offical.MasterListCfgGetOneData("0", "express_key")
			stringMap1["official_express_min_send_money"] = offical.MasterListCfgGetOneData("0", "express_min_send_money")   //最低预存款
			stringMap1["official_express_fee_send_money"] = offical.MasterListCfgGetOneData("0", "express_fee_send_money")   //充值预存款手续费
			stringMap1["official_express_first_pay_bili"] = offical.MasterListCfgGetOneData("0", "express_first_pay_bili")   //首重加价
			stringMap1["official_express_second_pay_bili"] = offical.MasterListCfgGetOneData("0", "express_second_pay_bili") //续重加价
			cache.SetEx(key1, utils.SerializeStr(stringMap1), 300)
		}
		for k, v := range stringMap1 {
			stringMap[k] = v
		}
	}
	return stringMap
}
