package consume

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/cc14514/go-geoip2"
	geoip2db "github.com/cc14514/go-geoip2-db"
	"github.com/streadway/amqp"
	"net"
	"strings"
	"time"
)

var data []*model.MasterAreaVisitsFlows

func ZhiOsUserVisitIpAddressConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>ZhiOsUserVisitIpAddressConsume<<<<<<<<<<<<<<<<<<<<<<<")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	geoIp2db, _ := geoip2db.NewGeoipDbByStatik()
	defer geoIp2db.Close()

	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(100)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleUserVisitIpAddress(res.Body, geoIp2db)
			//_ = res.Reject(false)
			_ = res.Ack(true)
			if len(data) >= 100 {
				db.BatchAddMasterAreaVisitsFlows(db.DataDb, data)
				data = []*model.MasterAreaVisitsFlows{}
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleUserVisitIpAddress(msg []byte, geoIp2db *geoip2.DBReader) error {
	now := time.Now()
	today := now.Format("2006-01-02")
	var tmpString string
	err := json.Unmarshal(msg, &tmpString)
	if err != nil {
		fmt.Println(err.Error())
		return err
	}
	var msgStruct md.ZhiOsUserVisitIpAddressMessage
	err = json.Unmarshal([]byte(tmpString), &msgStruct)
	if err != nil {
		fmt.Println(err.Error())
		return err
	}

	if msgStruct.MasterId == "" {
		cacheKeyForUrl := fmt.Sprintf(md.ZhiOsMasterIdForUrlHashMapCacheKey)
		get, _ := cache.HGetString(cacheKeyForUrl, msgStruct.Url)
		if get == "" {
			msgStruct.MasterId = GetWebSiteDomainMasterId(msgStruct.Url)
			utils.FilePutContents("GetWebSiteDomainMasterId", utils.SerializeStr(msgStruct))
			cache.HSet(cacheKeyForUrl, msgStruct.Url, msgStruct.MasterId)
			cache.Expire(cacheKeyForUrl, md.ZhiOsUserVisitIpAddressHashMapCacheTime)
		} else {
			msgStruct.MasterId = get
		}
	}

	//1、判断ip是否已统计
	cacheKey := fmt.Sprintf(md.ZhiOsUserVisitIpAddressHashMapCacheKey, msgStruct.MasterId, today)
	//get, _ := cache.HGetString(cacheKey, msgStruct.Ip)
	//if get == "" {
	if true {
		//2、分析ip归属地
		countryName, provinceId, cityId, provinceName, cityName := getIpAddress(geoIp2db, msgStruct)
		if provinceId != "" || cityId != "" || countryName != "" || provinceName != "" || cityName != "" {
			//3、插入`master_area_visits_flows` 表
			if provinceName == "" {
				provinceName = msgStruct.Province
			}
			if cityName == "" {
				cityName = msgStruct.City
			}

			data = append(data, &model.MasterAreaVisitsFlows{
				Ip:           msgStruct.Ip,
				MasterId:     utils.StrToInt(msgStruct.MasterId),
				Date:         today,
				CountryName:  countryName,
				ProvinceName: provinceName,
				ProvinceId:   provinceId,
				CityName:     cityName,
				CityId:       cityId,
				Pvd:          msgStruct.Pvd,
				CreateAt:     now,
				UpdateAt:     now,
			})
			//db.MasterAreaVisitsFlowsInsert(db.DataDb, )
		}
		//4、加入到缓存map中
		cache.HSet(cacheKey, msgStruct.Ip, msgStruct.MasterId)
		cache.Expire(cacheKey, md.ZhiOsUserVisitIpAddressHashMapCacheTime)
	}
	return nil
}

func getIpAddress(geoIp2db *geoip2.DBReader, message md.ZhiOsUserVisitIpAddressMessage) (countryName, provinceId, cityId, provinceName, cityName string) {
	record, _ := geoIp2db.City(net.ParseIP(message.Ip))
	if record.Country.Names != nil && record.Subdivisions != nil && record.City.Names != nil {
		message.Country = record.Country.Names["zh-CN"]
		message.Province = record.Subdivisions[0].Names["zh-CN"]
		message.City = record.City.Names["zh-CN"]
	}

	countryName = message.Country
	if countryName == "中国" && message.Province != "" {
		if message.Province == "闽" {
			message.Province = "福建"
		}
		province, err := db.ProvinceGetOneByName(message.Province)
		if err != nil {
			return "", "", "", "", ""
		}
		provinceId = province.Id
		provinceName = province.Name

		if message.City != "" {
			city, err := db.CityGetOneByName(message.City)
			if err != nil {
				return "", "", "", "", ""
			}
			cityId = city.Id
			cityName = city.Name
		}
	}
	return
}

// 获取指定类型的域名对应的masterId：admin、wap、api
func GetWebSiteDomainMasterId(host string) string {
	obj := new(model.UserAppDomain)
	has, err := db.Db.Where("domain=?", host).Get(obj)
	if err != nil || !has {
		hostList := strings.Split(host, ":")
		has, err = db.Db.Where("domain=?", hostList[0]).Get(obj)
		if err != nil || !has {
			return ""
		}
	}
	return utils.AnyToString(obj.Uuid)
}
