package model

import (
	"time"
)

type MallGoods struct {
	GoodsId                 int       `json:"goods_id" xorm:"not null pk autoincr comment('商品id') INT(11)"`
	GoodsType               int       `json:"goods_type" xorm:"not null default 1 comment('商品类型：1实物商品') TINYINT(1)"`
	Title                   string    `json:"title" xorm:"not null default '' comment('商品标题') VARCHAR(1024)"`
	CategoryId              int       `json:"category_id" xorm:"not null default 0 comment('类目ID') INT(11)"`
	ShareText               string    `json:"share_text" xorm:"not null default '' comment('分享描述') VARCHAR(2048)"`
	ImageList               string    `json:"image_list" xorm:"comment('商品图json') TEXT"`
	CommissionRate          string    `json:"commission_rate" xorm:"not null default 0.0000 comment('佣金比例') DECIMAL(6,4)"`
	IcbcCommissionRate      string    `json:"icbc_commission_rate" xorm:"not null default 0.0000 comment('佣金比例') DECIMAL(6,4)"`
	IntegralRate            string    `json:"integral_rate" xorm:"not null default 0.0000 comment('积分分佣比例') DECIMAL(6,4)"`
	Price                   string    `json:"price" xorm:"not null default 0.00 comment('价格') DECIMAL(12,2)"`
	BuyNeedDeductBili       string    `json:"buy_need_deduct_bili" xorm:"not null default 0.00 comment('抵扣券最大比例') DECIMAL(12,2)"`
	LinePrice               string    `json:"line_price" xorm:"not null default 0.00 comment('划线价') DECIMAL(12,2)"`
	VipDiscountType         int       `json:"vip_discount_type" xorm:"not null default 1 comment('会员优惠方式：1打折 2减价 3指定价格') TINYINT(3)"`
	VipDiscountPrice        string    `json:"vip_discount_price" xorm:"comment('会员优惠价信息') TEXT"`
	Stock                   int       `json:"stock" xorm:"not null default 0 comment('库存') INT(255)"`
	Weight                  string    `json:"weight" xorm:"not null default 0.0000 comment('重量') DECIMAL(12,4)"`
	ShippingType            int       `json:"shipping_type" xorm:"not null default 1 comment('配送方式：1快递') TINYINT(1)"`
	ShippingFeeType         int       `json:"shipping_fee_type" xorm:"not null default 1 comment('运费计费方式：1统一运费 2运费模板') TINYINT(1)"`
	ShippingTplId           int       `json:"shipping_tpl_id" xorm:"not null default 0 comment('运费模板id') INT(11)"`
	ShippingFee             string    `json:"shipping_fee" xorm:"not null default 0.00 comment('统一运费的值') DECIMAL(12,2)"`
	SaleState               int       `json:"sale_state" xorm:"not null default 1 comment('销售状态：1上架（立即销售）2定时销售 3放入仓库') TINYINT(1)"`
	SaleStartTime           time.Time `json:"sale_start_time" xorm:"comment('定时开售时间') DATETIME"`
	IsLimitBuy              int       `json:"is_limit_buy" xorm:"not null default 0 comment('是否限购：0否 1是') TINYINT(1)"`
	LimitBuyData            string    `json:"limit_buy_data" xorm:"not null default '' comment('限购数据：type：1：终身限购n件 2：按周期x限购y件') VARCHAR(5012)"`
	CreateTime              time.Time `json:"create_time" xorm:"created not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	UpdateTime              time.Time `json:"update_time" xorm:"updated not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	OfficialProperty        string    `json:"official_property" xorm:"not null comment('官方参数json') TEXT"`
	CustomProperty          string    `json:"custom_property" xorm:"not null comment('自定义参数json') TEXT"`
	Spe                     string    `json:"spe" xorm:"not null default '' comment('所有规格属性json') VARCHAR(5012)"`
	SpeImages               string    `json:"spe_images" xorm:"not null default '' comment('第一组规格值对应的图片json') VARCHAR(1024)"`
	Detail                  string    `json:"detail" xorm:"comment('商品详情') TEXT"`
	VipDiscountSwitch       int       `json:"vip_discount_switch" xorm:"not null default 0 comment('是否开启会员价：0关闭 1开启') TINYINT(1)"`
	Sort                    int       `json:"sort" xorm:"not null default 0 comment('序号') INT(11)"`
	ActivityType            int       `json:"activity_type" xorm:"not null default 0 comment('参与的活动：0未参加活动 1拼团活动 2秒杀活动 3砍价活动 4超级拼团 5会员升级礼包') TINYINT(3)"`
	SaleCount               int       `json:"sale_count" xorm:"not null default 0 comment('销量') INT(11)"`
	IsSpeImageOn            int       `json:"is_spe_image_on" xorm:"not null default 0 comment('是否开启规格图片：0否 1是') TINYINT(1)"`
	IsSpeImageInDetail      int       `json:"is_spe_image_in_detail" xorm:"not null default 0 comment('规格图是否显示在详情页') TINYINT(1)"`
	DeleteAt                time.Time `json:"delete_at" xorm:"comment('商品删除字段，不为null时，表示删除') DATETIME"`
	Service                 string    `json:"service" xorm:"comment('支持的服务json，id数组') VARCHAR(512)"`
	IsVirtualCoin           int       `json:"is_virtual_coin" xorm:"comment('是否支持虚拟币') TINYINT(1)"`
	VirtualCoinId           int       `json:"virtual_coin_id" xorm:"comment('支持支付的虚拟币id') INT(11)"`
	RepurchaseDiscount      string    `json:"repurchase_discount" xorm:"default 0.00 comment('复购折扣') DECIMAL(12,2)"`
	UserLevel               int       `json:"user_level" xorm:"default 0 comment('升级等级') INT(11)"`
	UserLevelDay            int       `json:"user_level_day" xorm:"default 0 comment('升级等级天数 0为永久') INT(11)"`
	IsHasRepurchase         int       `json:"is_has_repurchase" xorm:"default 0 comment('是否有复购') INT(1)"`
	RepurchaseType          int       `json:"repurchase_type" xorm:"default 0 comment('复购类型 0购买相同商品为复购 1购买商品赠送相同会员等级为复购') INT(11)"`
	IsHasUserLevel          int       `json:"is_has_user_level" xorm:"default 0 comment('是否有会员权益') INT(1)"`
	UserLevelGivenStatus    int       `json:"user_level_given_status" xorm:"default 0 comment('会员礼包赠送权益订单状态') INT(11)"`
	UserLevelUpTimeType     int       `json:"user_level_up_time_type" xorm:"default 0 comment('升级礼包升级方式 0过期时间延长  1当前时间延长') INT(11)"`
	UserLevelCanRefund      int       `json:"user_level_can_refund" xorm:"default 0 comment('会员礼包是否能退货') INT(1)"`
	SeckillStock            int       `json:"seckill_stock" xorm:"default 0 comment('秒杀库存（总）') INT(11)"`
	VideoUrl                string    `json:"video_url" xorm:"not null default '' comment('视频链接') VARCHAR(255)"`
	VideoHeight             string    `json:"video_height" xorm:"not null default '' comment('视频高度') VARCHAR(100)"`
	GoodsSourceUrl          string    `json:"goods_source_url" xorm:"not null default '' comment('商品来源链接') TEXT"`
	SalesStoreName          string    `json:"sales_store_name" xorm:"not null default '' comment('经销商名称') VARCHAR(100)"`
	Pvd                     int       `json:"pvd" xorm:"default 1 comment('来源：1:商家自己发布，2：来自于供应商') TINYINT(1)"`
	SupplierGoodsId         int64     `json:"supplier_goods_id" xorm:"comment('供应商的商品id') INT(11)"`
	SupplierMerchantId      int       `json:"supplier_merchant_id" xorm:"comment('供应商id') INT(11)"`
	SupplierMerchantName    string    `json:"supplier_merchant_name" xorm:"comment('供应商名称') VARCHAR(255)"`
	SupplierCategoryId      int       `json:"supplier_category_id" xorm:"comment('供应商分类id') INT(11)"`
	SupplierGoodsUpdateTime string    `json:"supplier_goods_update_time" xorm:"default '' VARCHAR(100)"`
	OriginalPrice           string    `json:"original_price" xorm:"comment('原价（指站长库的价格）') DECIMAL(12,4)"`
	AddPrice                string    `json:"add_price" xorm:"comment('加价（指在原价上加多少钱）') DECIMAL(10,2)"`
	SupplierMerchantAvatar  string    `json:"supplier_merchant_avatar" xorm:"comment('供应链商家头像') VARCHAR(255)"`
	AddPriceRule            string    `json:"add_price_rule" xorm:"comment('加价规则json') TEXT"`
	SupplierGoodsCode       string    `json:"supplier_goods_code" xorm:"comment('供应商的商品编号') VARCHAR(255)"`
	Msg                     string    `json:"msg" xorm:"comment('商品的错误信息或其他内容') TEXT"`
	RewardGuessCardNum      int       `json:"reward_guess_card_num" xorm:"default 0 comment('奖励拆猜卡数量') INT(11)"`
	LimitJoinType           string    `json:"limit_join_type" xorm:"not null default '' comment('条件限制json') VARCHAR(1024)"`
	LevelType               string    `json:"level_type" xorm:"not null default '' comment('等级限制json') VARCHAR(1024)"`
	AgentRule               string    `json:"agent_rule" xorm:"not null default '' comment('等级限制json') VARCHAR(5012)"`
	IntegralRule            string    `json:"integral_rule" xorm:"not null default '' comment('等级限制json') VARCHAR(500)"`
	IntegralPayType         int       `json:"integral_pay_type" xorm:"default 0 comment('积分支付方式 0固定积分金额  1积分不够余额付') INT(1)"`
	IsVirtualGoods          int       `json:"is_virtual_goods" xorm:"default 0 comment('') INT(1)"`
	RewardVirtualCoinId     int       `json:"reward_virtual_coin_id" xorm:"default 0 comment('') INT(11)"`
	RewardVirtualUserLv     string    `json:"reward_virtual_user_lv" xorm:"not null comment('自定义参数json') TEXT"`
	BaseCommission          string    `json:"base_commission" xorm:"comment('积分兑换模式基数') DECIMAL(12,4)"`
	PlatformCostPrice       string    `json:"platform_cost_price" xorm:"comment('平台成本价') DECIMAL(12,4)"`
	GroupRuleData           string    `json:"group_rule_data" xorm:" comment('拼团数据 json') TEXT"`
	BelongCity              string    `json:"belong_city" xorm:"not null default '' comment('所属城市 大陆mainland  澳门 macao') VARCHAR(255)"`
	VirtualGoodsInfo        string    `json:"virtual_goods_info" xorm:"not null default '' comment('') VARCHAR(1024)"`
	ConsignForSaleInfo      string    `json:"consign_for_sale_info" xorm:"not null default '' comment('') VARCHAR(1024)"`
	ConsignForSaleAddCart   int       `json:"consign_for_sale_add_cart" xorm:"default 0 comment('') INT(11)"`
	IsJuShuiTan             int       `json:"is_ju_shui_tan" xorm:"default 0 comment('') INT(11)"`
}
