package model

import (
	"time"
)

type MallOfficialGoods struct {
	GoodsId               int       `json:"goods_id" xorm:"not null pk autoincr comment('商品id') INT(11)"`
	GoodsType             int       `json:"goods_type" xorm:"not null default 1 comment('商品类型：1实物商品') TINYINT(1)"`
	Title                 string    `json:"title" xorm:"not null default '' comment('商品标题') VARCHAR(1024)"`
	CategoryId            int       `json:"category_id" xorm:"not null default 0 comment('类目ID') INT(11)"`
	ShareText             string    `json:"share_text" xorm:"not null default '' comment('分享描述') VARCHAR(2048)"`
	ImageList             string    `json:"image_list" xorm:"comment('商品图json') TEXT"`
	Price                 string    `json:"price" xorm:"not null default 0.00 comment('价格（original_price+add_price）站长实收数值') DECIMAL(12,2)"`
	Discount              string    `json:"discount" xorm:"default 10.00 comment('折扣，为10无折扣') DECIMAL(6,2)"`
	DiscountPrice         string    `json:"discount_price" xorm:"not null default 0.00 comment('折扣价') DECIMAL(12,2)"`
	VipDiscountType       int       `json:"vip_discount_type" xorm:"not null default 1 comment('会员优惠方式：0原价 1会员 2折扣') TINYINT(3)"`
	VipPrice              string    `json:"vip_price" xorm:"comment('会员价，会员等级对应的价格值json:格式为：[{"level_id":1,"value":10}]') TEXT"`
	Stock                 int       `json:"stock" xorm:"not null default 0 comment('库存') INT(255)"`
	Weight                string    `json:"weight" xorm:"not null default 0.0000 comment('重量') DECIMAL(12,4)"`
	ShippingType          int       `json:"shipping_type" xorm:"not null default 1 comment('配送方式：1快递') TINYINT(1)"`
	ShippingFeeType       int       `json:"shipping_fee_type" xorm:"not null default 1 comment('运费计费方式：1统一运费 2运费模板') TINYINT(1)"`
	ShippingTplId         int       `json:"shipping_tpl_id" xorm:"not null default 0 comment('运费模板id') INT(11)"`
	ShippingFee           string    `json:"shipping_fee" xorm:"not null default 0.00 comment('统一运费的值') DECIMAL(12,2)"`
	SaleState             int       `json:"sale_state" xorm:"not null default 1 comment('销售状态：1上架（立即销售）2定时销售 3放入仓库') TINYINT(1)"`
	SaleStartTime         time.Time `json:"sale_start_time" xorm:"comment('定时开售时间') DATETIME"`
	IsLimitBuy            int       `json:"is_limit_buy" xorm:"not null default 0 comment('是否限购：0否 1是') TINYINT(1)"`
	LimitBuyData          string    `json:"limit_buy_data" xorm:"not null default '' comment('限购数据：type：1：终身限购n件 2：按周期x限购y件') VARCHAR(5012)"`
	CreateTime            time.Time `json:"create_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	UpdateTime            time.Time `json:"update_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	OfficialProperty      string    `json:"official_property" xorm:"not null comment('官方参数json') TEXT"`
	CustomProperty        string    `json:"custom_property" xorm:"not null comment('自定义参数json') TEXT"`
	Spe                   string    `json:"spe" xorm:"not null default '' comment('所有规格属性json') VARCHAR(5012)"`
	SpeImages             string    `json:"spe_images" xorm:"not null default '' comment('第一组规格值对应的图片json') VARCHAR(1024)"`
	Detail                string    `json:"detail" xorm:"comment('商品详情') TEXT"`
	Sort                  int       `json:"sort" xorm:"not null default 0 comment('序号') INT(11)"`
	ActivityType          int       `json:"activity_type" xorm:"not null default 0 comment('参与的活动：0未参加活动 1拼团活动 2秒杀活动 3砍价活动 4超级拼团') TINYINT(3)"`
	SaleCount             int       `json:"sale_count" xorm:"not null default 0 comment('销量') INT(11)"`
	IsSpeImageOn          int       `json:"is_spe_image_on" xorm:"not null default 0 comment('是否开启规格图片：0否 1是') TINYINT(1)"`
	IsSpeImageInDetail    int       `json:"is_spe_image_in_detail" xorm:"not null default 0 comment('规格图是否显示在详情页') TINYINT(1)"`
	DeleteAt              time.Time `json:"delete_at" xorm:"comment('商品删除字段，不为null时，表示删除') DATETIME"`
	Service               string    `json:"service" xorm:"comment('支持的服务json，id数组') VARCHAR(512)"`
	IsVirtualCoin         int       `json:"is_virtual_coin" xorm:"comment('是否支持虚拟币') TINYINT(1)"`
	VirtualCoinId         int       `json:"virtual_coin_id" xorm:"comment('支持支付的虚拟币id') INT(11)"`
	StoreId               int       `json:"store_id" xorm:"not null comment('店铺id') INT(11)"`
	IsSingleSku           int       `json:"is_single_sku" xorm:"not null default 1 comment('是否单规格，0：否，1是') TINYINT(1)"`
	CommissionRate        string    `json:"commission_rate" xorm:"not null default 0.0000 comment('佣金比例') DECIMAL(6,4)"`
	AuditState            int       `json:"audit_state" xorm:"default 1 comment('审核状态：1未审核 2通过 3失败') TINYINT(1)"`
	AuditRemark           string    `json:"audit_remark" xorm:"comment('审核备注') VARCHAR(255)"`
	StockDeductionWay     int       `json:"stock_deduction_way" xorm:"default 1 comment('库存减扣方式  1：拍下减库存，') TINYINT(1)"`
	IsReturnWithoutReason int       `json:"is_return_without_reason" xorm:"default 0 comment('是否7天无理由退货（0否，1是）') TINYINT(1)"`
	IsManufactorDelivery  int       `json:"is_manufactor_delivery" xorm:"default 0 comment('是否厂家配送（0:否，1是）') TINYINT(1)"`
	IsOfficialFidelity    int       `json:"is_official_fidelity" xorm:"default 0 comment('是官方保真（0:否，1:是）') TINYINT(1)"`
	Pvd                   int       `json:"pvd" xorm:"comment('来源：1:商家自己发布，2：来自于供应商') TINYINT(1)"`
	SupplierGoodsId       int       `json:"supplier_goods_id" xorm:"comment('供应商的商品id') INT(11)"`
	SupplierMerchantId    int       `json:"supplier_merchant_id" xorm:"comment('供应商id') INT(11)"`
	SupplierMerchantName  string    `json:"supplier_merchant_name" xorm:"comment('供应商名称') VARCHAR(255)"`
	SupplierCategoryId    int       `json:"supplier_category_id" xorm:"comment('供应商分类id') INT(11)"`
	OriginalPrice         string    `json:"original_price" xorm:"comment('原价（指供应链的价格）') DECIMAL(12,4)"`
	AddPrice              string    `json:"add_price" xorm:"comment('加价（指在原价上加多少钱）') DECIMAL(10,2)"`
	LinePrice             string    `json:"line_price" xorm:"comment('指导价（指供应链的价格）') DECIMAL(12,4)"`
}
