package svc

import (
	"applet/app/utils"
	"applet/app/utils/logx"
	supplySvc "applet/mall/curl_supply/svc"
	"applet/mall/db"
	"applet/mall/db/model"
	"applet/mall/md"
	utils2 "applet/mall/utils"
	"errors"
	"github.com/gin-gonic/gin"
	"xorm.io/xorm"
)

func AddOfficialGoods(c *gin.Context, engine *xorm.Engine, goods *md.OfficialGoods, conditions *md.OfficialGoodsConditions) error {
	exist, err2 := engine.Where("supplier_goods_id = ?", goods.ID).Exist(new(model.MallGoods))
	if exist {
		return errors.New("该商品已存在无需重复拉取")
	}
	details, err2 := supplySvc.CurlSupplyGoodsDetails(c, goods.ID)
	if err2 != nil {
		return err2
	}
	if details.BaseGoods.GoodsID <= 0 {
		return errors.New("错误的供应链商品id")
	}
	if details.BaseGoods.MerchantID <= 0 {
		return errors.New("错误的供应链站长id")
	}
	for _, sku := range details.Skus {
		if sku.GoodsID <= 0 || sku.SkuId <= 0 {
			return errors.New("错误的供应链商品SkuId")
		}
	}
	if details.BaseGoods.SaleState != 1 {
		return errors.New("该商品不是销售中状态！")
	}
	session := engine.NewSession()
	defer session.Close()
	session.Begin()
	var officialGoods model.MallGoods
	officialGoods.CategoryId = utils.StrToInt(conditions.CategoryId)
	officialGoods.Title = details.BaseGoods.Title
	//officialGoods.AuditState = 2
	//imgStringSlice := make([]string, 0)
	//imgStringSlice = append(imgStringSlice, goods.Image)
	officialGoods.AddPriceRule = utils.SerializeStr(conditions)
	var goodsConditions md.Conditions
	goodsConditions.AddPriceBase = conditions.AddPriceBase
	goodsConditions.AddPriceNum = conditions.AddPriceNum
	goodsConditions.AddPriceType = conditions.AddPriceType
	goodsConditions.NumType = conditions.NumType

	officialGoods.ImageList = utils.SerializeStr(details.BaseGoods.ImageListURL)
	officialGoods.GoodsType = details.BaseGoods.GoodsType
	if details.BaseGoods.DetailURL == nil {
		details.BaseGoods.DetailURL = make([]string, 0)
	}
	officialGoods.Detail = utils.SerializeStr(details.BaseGoods.DetailURL)
	officialGoods.LinePrice = goods.LinePrice
	officialGoods.SupplierMerchantId = details.BaseGoods.MerchantID
	officialGoods.SupplierGoodsId = details.BaseGoods.GoodsID
	officialGoods.SupplierCategoryId = details.BaseGoods.CategoryID
	officialGoods.SupplierMerchantName = details.MallName
	officialGoods.AddPrice = calculateAddPriceNum(goods.Price, goods.LinePrice, &goodsConditions)
	officialGoods.SaleState = details.BaseGoods.SaleState
	officialGoods.Stock = details.BaseGoods.Stock
	officialGoods.Pvd = 2
	officialGoods.Weight = goods.Weight
	officialGoods.SaleCount = details.BaseGoods.SaleCount
	officialGoods.OriginalPrice = details.BaseGoods.Price
	officialGoods.Price = utils.AnyToString(utils.StrToFloat64(officialGoods.AddPrice) + utils.StrToFloat64(officialGoods.OriginalPrice))
	officialGoods.Spe = utils.SerializeStr(details.BaseGoods.Spe)
	officialGoods.SpeImages = utils.SerializeStr(details.BaseGoods.SpeImagesURL)
	officialGoods.IsSpeImageInDetail = details.BaseGoods.IsSpeImageInDetail
	officialGoods.IsSpeImageOn = details.BaseGoods.IsSpeImageOn
	officialGoods.SaleStartTime = utils2.String2Time(details.BaseGoods.SaleStartTime)
	officialGoods.VipDiscountType = 1
	officialGoods.VipDiscountPrice = "0"
	officialGoods.SupplierGoodsCode = details.BaseGoods.GoodsCode
	officialGoods.CustomProperty = utils.SerializeStr(details.BaseGoods.CustomProperty)

	//todo 是否需要同步运费模板！！
	officialGoods.ShippingFee = details.BaseGoods.ShippingFee
	officialGoods.ShippingFeeType = details.BaseGoods.ShippingFeeType
	officialGoods.ShippingTplId = details.BaseGoods.ShippingTplID
	officialGoods.ShippingType = details.BaseGoods.ShippingType

	officialGoods.SupplierMerchantAvatar = details.MallLogo
	_, err := session.InsertOne(&officialGoods)
	if err != nil {
		session.Rollback()
		return err
	}
	if officialGoods.GoodsId == 0 {
		session.Rollback()
		return errors.New("数据库操作失败！")
	}
	skuConditionsMap := make(map[string]*md.OfficialSkuConditions)
	for _, condition := range conditions.SkusConditions {
		skuConditionsMap[condition.SkuCode] = condition
	}
	for _, sku := range goods.SkuList {
		var officialSku model.MallSku
		officialSku.GoodsId = officialGoods.GoodsId
		officialSku.SaleCount = sku.SaleCount
		officialSku.Sku = sku.Sku
		officialSku.SupplierGoodsId = sku.GoodsID
		officialSku.SupplierSkuId = sku.SkuID
		officialSku.Indexes = sku.Indexes
		officialSku.Weight = sku.Weight
		officialSku.Stock = sku.Stock

		var skuConditions md.Conditions
		skuConditions.AddPriceBase = conditions.AddPriceBase
		skuConditions.AddPriceNum = conditions.AddPriceNum
		skuConditions.AddPriceType = conditions.AddPriceType
		skuConditions.NumType = conditions.NumType
		if conditions.IsSkuAddPrice == "1" {
			officialSkuConditions, ok := skuConditionsMap[sku.SkuCode]
			if !ok {
				session.Rollback()
				return errors.New("SKU编码对应的加价信息获取失败或SKU编码不一致，无法加价！")
			}
			skuConditions.AddPriceBase = officialSkuConditions.AddPriceBase
			skuConditions.AddPriceNum = officialSkuConditions.AddPriceNum
			skuConditions.AddPriceType = officialSkuConditions.AddPriceType
			skuConditions.NumType = officialSkuConditions.NumType
		}

		officialSku.OriginalPrice = sku.Price
		officialSku.AddPrice = calculateAddPriceNum(sku.Price, sku.LinePrice, &skuConditions)
		officialSku.Price = utils.AnyToString(utils.StrToFloat64(officialSku.AddPrice) + utils.StrToFloat64(officialSku.OriginalPrice))

		officialSku.SupplierSkuId = sku.SkuID
		officialSku.SupplierGoodsId = sku.GoodsID
		officialSku.SupplierSkuCode = sku.SkuCode
		officialSku.CloudChainSkuImgUrl = sku.CloudChainSkuImgUrl
		one, err := session.InsertOne(&officialSku)
		if err != nil || one == 0 {
			session.Rollback()
			return err
		}
		err = db.AddOrUpdateMallSkuAddPriceRule(session, utils.SerializeStr(&skuConditions), details.BaseGoods.GoodsCode+"-"+sku.SkuCode)
		if err != nil {
			logx.Error(err)
			session.Rollback()
			return err
		}
	}
	err = session.Commit()
	if err != nil {
		session.Rollback()
		return err
	}
	return nil
}

func calculateAddPriceNum(originalPrice string, linePrice string, conditions *md.Conditions) string {
	addPrice := "0.0"
	if conditions.AddPriceType == "1" {
		if conditions.AddPriceBase == "1" {
			addPrice = utils.AnyToString(utils.StrToFloat64(conditions.AddPriceNum))
		} else if conditions.AddPriceBase == "2" {
			profits := utils.StrToFloat64(linePrice) - utils.StrToFloat64(originalPrice)
			if profits < 0 {
				profits = utils.StrToFloat64(originalPrice) - utils.StrToFloat64(linePrice)
			}
			if profits < 0 {
				profits = 0
			}
			addPrice = utils.AnyToString(profits + utils.StrToFloat64(conditions.AddPriceNum))
		}
	} else if conditions.AddPriceType == "2" {
		if conditions.AddPriceBase == "1" {
			addPriceRate := utils.StrToFloat64(conditions.AddPriceNum) / 100
			addPrice = utils.AnyToString(utils.StrToFloat64(originalPrice) * addPriceRate)
		} else if conditions.AddPriceBase == "2" {
			addPriceRate := utils.StrToFloat64(conditions.AddPriceNum) / 100
			profits := utils.StrToFloat64(linePrice) - utils.StrToFloat64(originalPrice)
			if profits < 0 {
				profits = utils.StrToFloat64(originalPrice) - utils.StrToFloat64(linePrice)
			}
			if profits < 0 {
				profits = 0
			}
			addPrice = utils.AnyToString(profits * addPriceRate)
		}
	}
	return addPrice
}
