package svc

import (
	"applet/mall/curl_supply/md"
	db2 "applet/supply/db"
	"github.com/gin-gonic/gin"
	"xorm.io/xorm"

	"applet/app/cfg"
	"applet/app/db"

	"applet/app/utils"
	"applet/app/utils/cache"
)

// 单挑记录获取
func SysCfgGet(c *gin.Context, key string) string {
	res := SysCfgFind(c, key)
	if _, ok := res[key]; !ok {
		return ""
	}
	return res[key]
}

// 多条记录获取
func SysCfgFind(c *gin.Context, keys ...string) map[string]string {
	var e *xorm.Engine
	if c == nil {
		e = db.Db
	} else {
		e = db.DBs[c.GetString("mid")]
	}
	res := map[string]string{}
	err := cache.GetJson(md.KEY_SYS_CFG_CACHE, &res)
	if err != nil || len(res) == 0 {
		cfgList, _ := db.SysCfgGetAll(e)
		if cfgList == nil {
			return nil
		}
		for _, v := range *cfgList {
			res[v.Key] = v.Val
		}
		// 先不设置缓存
		// cache.SetJson(md.KEY_SYS_CFG_CACHE, res, 60)
	}
	if len(keys) == 0 {
		return res
	}
	tmp := map[string]string{}
	for _, v := range keys {
		if val, ok := res[v]; ok {
			tmp[v] = val
		} else {
			tmp[v] = ""
		}
	}
	return tmp
}

// 清理系统配置信息
func SysCfgCleanCache() {
	cache.Del(md.KEY_SYS_CFG_CACHE)
}

// 写入系统设置
func SysCfgSet(c *gin.Context, key, val, memo string) bool {
	cfg, err := db.SysCfgGetOne(db.DBs[c.GetString("mid")], key)
	if err != nil || cfg == nil {
		return db.SysCfgInsert(db.DBs[c.GetString("mid")], key, val, memo)
	}
	if memo != "" && cfg.Memo != memo {
		cfg.Memo = memo
	}
	SysCfgCleanCache()
	return db.SysCfgUpdate(db.DBs[c.GetString("mid")], key, val, cfg.Memo)
}

// 多条记录获取
func SysCfgFindByIds(eg *xorm.Engine, keys ...string) map[string]string {
	key := utils.Md5(eg.DataSourceName() + md.KEY_SYS_CFG_CACHE)
	res := map[string]string{}
	c, ok := cfg.MemCache.Get(key).(map[string]string)
	if !ok || len(c) == 0 {
		cfgList, _ := db2.DbsSysCfgGetAll(eg)
		if cfgList == nil {
			return nil
		}
		for _, v := range *cfgList {
			res[v.Key] = v.Val
		}
		cfg.MemCache.Put(key, res, 10)
	} else {
		res = c
	}
	if len(keys) == 0 {
		return res
	}
	tmp := map[string]string{}
	for _, v := range keys {
		if val, ok := res[v]; ok {
			tmp[v] = val
		} else {
			tmp[v] = ""
		}
	}
	return tmp
}
