package utils

import "time"

// 自定义时间类型 用于解决json解析错误问题
type LocalTime time.Time

const TimeFormat = "2006-01-02 15:04:05"

// 实现 json解析（反序列化）接口，调用c.ShouldBindJSON()方法时，ShouldBindJSON()方法会调用
func (t *LocalTime) UnmarshalJSON(data []byte) (err error) {
	// 空值不进行解析
	if len(data) == 2 {
		*t = LocalTime(time.Time{})
		return
	}

	// 指定解析的格式
	now, err := time.Parse(`"`+TimeFormat+`"`, string(data))
	*t = LocalTime(now)
	return
}

// 实现 json（序列化）接口
func (t LocalTime) MarshalJSON() ([]byte, error) {
	b := make([]byte, 0, len(TimeFormat)+2)
	b = append(b, '"')
	b = time.Time(t).AppendFormat(b, TimeFormat)
	b = append(b, '"')
	return b, nil
}
