package model

import (
	"time"
)

type MallGoods struct {
	GoodsId            int64     `json:"goods_id" xorm:"not null BIGINT(20)"`
	GoodsCode          string    `json:"goods_code" xorm:"not null default '' comment('商品編碼') VARCHAR(255)"`
	GoodsType          int       `json:"goods_type" xorm:"not null default 1 comment('商品类型：1实物商品') TINYINT(1)"`
	Title              string    `json:"title" xorm:"not null default '' comment('商品标题') VARCHAR(1024)"`
	CategoryId         int       `json:"category_id" xorm:"not null default 0 comment('类目ID') INT(11)"`
	ImageList          string    `json:"image_list" xorm:"comment('商品图json') TEXT"`
	Stock              int       `json:"stock" xorm:"not null default 0 comment('库存') INT(255)"`
	ShippingType       int       `json:"shipping_type" xorm:"not null default 1 comment('配送方式：1快递') TINYINT(1)"`
	ShippingFeeType    int       `json:"shipping_fee_type" xorm:"not null default 1 comment('运费计费方式：1统一运费 2运费模板') TINYINT(1)"`
	ShippingTplId      int       `json:"shipping_tpl_id" xorm:"not null default 0 comment('运费模板id') INT(11)"`
	ShippingFee        string    `json:"shipping_fee" xorm:"not null default 0.00 comment('统一运费的值') DECIMAL(12,2)"`
	SaleState          int       `json:"sale_state" xorm:"not null default 0 comment('销售状态：-1审核拒绝 0审核中 1销售中 2已售罄 3放入仓库 4下架') TINYINT(1)"`
	SaleStartTime      time.Time `json:"sale_start_time" xorm:"comment('定时开售时间') DATETIME"`
	CreateTime         time.Time `json:"create_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	UpdateTime         time.Time `json:"update_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	CustomProperty     string    `json:"custom_property" xorm:"not null comment('自定义参数json') TEXT"`
	Spe                string    `json:"spe" xorm:"not null default '' comment('所有规格属性json') VARCHAR(5012)"`
	SpeImages          string    `json:"spe_images" xorm:"not null default '' comment('第一组规格值对应的图片json') VARCHAR(1024)"`
	Detail             string    `json:"detail" xorm:"comment('商品详情') TEXT"`
	SaleCount          int       `json:"sale_count" xorm:"not null default 0 comment('销量') INT(11)"`
	IsSpeImageOn       int       `json:"is_spe_image_on" xorm:"not null default 0 comment('是否开启规格图片：0否 1是') TINYINT(1)"`
	IsSpeImageInDetail int       `json:"is_spe_image_in_detail" xorm:"not null default 0 comment('规格图是否显示在详情页') TINYINT(1)"`
	DeleteAt           time.Time `json:"delete_at" xorm:"comment('商品删除字段，不为null时，表示删除') DATETIME"`
	Service            string    `json:"service" xorm:"comment('支持的服务json，id数组') VARCHAR(512)"`
	MerchantId         int       `json:"merchant_id" xorm:"comment('商家id') INT(11)"`
	ShippingTimeType   int       `json:"shipping_time_type" xorm:"not null default 3 comment('配送时间类型：1.当日发货 2.24小时内发货 3.48小时内发货 4自定义') TINYINT(1)"`
	ShippingTimeCustom string    `json:"shipping_time_custom" xorm:"comment('自定义发货时间的值（shipping_time_type=4时）') VARCHAR(50)"`
	ProfitRate         string    `json:"profit_rate" xorm:"default 0.00 comment('预计利润空间') DECIMAL(8,2)"`
	Price              string    `json:"price" xorm:"comment('sku中最小价') DECIMAL(10,2)"`
	AuditReason        string    `json:"audit_reason" xorm:"comment('审核失败原因') VARCHAR(255)"`
	PriceAddType       int       `json:"price_add_type" xorm:"not null default 1 comment('加价方式(1:按金额加 2:按比例加)') TINYINT(1)"`
	RatioBaseType      int       `json:"ratio_base_type" xorm:"not null default 1 comment('比例基数方式(1:按进货价 2:按市场指导价 3:按利润空间)') TINYINT(1)"`
	AddPriceValue      string    `json:"add_price_value" xorm:"not null default '0' comment('调整值(百分比或具体金额数值)') VARCHAR(50)"`
	PriceValueType     int       `json:"price_value_type" xorm:"not null default 1 comment('价格数值设置(1:整数 2:小数点后两位)') TINYINT(1)"`
	IsOpenAddPrice     int       `json:"is_open_add_price" xorm:"not null default 1 comment('是否开启商品加价') TINYINT(1)"`
	CloudChainGoodsId  string    `json:"cloud_chain_goods_id" xorm:"comment('云链产品id') VARCHAR(255)"`
	MinOrderQuantity   string    `json:"min_order_quantity" xorm:"comment('最小购买数') VARCHAR(255)"`
}
