package md

import "applet/supply/enum"

type AddGoodsReq struct {
	Base    MallBaseGoods   `json:"base" binding:"required" label:"商品基本信息"`        // 基本信息
	SkuList []*MallGoodsSku `json:"sku_list" binding:"required,gt=0" label:"商品规格"` // sku列表
	IsCopy  bool            `json:"is_copy,omitempty"`                             // 是否是复制商品
}

type MallBaseGoods struct {
	GoodsId         string                   `json:"goods_id,omitempty" label:"商品ID"` // 商品id,0或不传为新建
	GoodsCode       string                   `json:"goods_code,omitempty" label:"商品編碼"`
	GoodsType       enum.MallGoodsType       `json:"goods_type" default:"1" label:"商品类型"`             // 商品类型：1实体商品
	Title           string                   `json:"title" binding:"required,lte=200" label:"标题"`     // 商品标题
	CategoryId      int                      `json:"category_id" binding:"required" label:"商品类目"`     // 商品类目id
	ImageList       []string                 `json:"image_list" binding:"required,gt=0" label:"商品主图"` // 主图列表
	ImageListUrl    []string                 `json:"image_list_url" `                                 // 主图列表
	Service         []int                    `json:"service,omitempty" label:"支持服务"`
	ShippingType    enum.MallShippingType    `json:"shipping_type" default:"1" label:"配送方式"`                               // 配送方式
	ShippingFeeType enum.MallShippingFeeType `json:"shipping_fee_type" binding:"required,oneof=1 2" label:"运费计费方式"`        // 运费计费方式
	ShippingTplID   int                      `json:"shipping_tpl_id" binding:"required_if=ShippingFeeType 2" label:"运费模板"` //运费模板id
	ShippingFee     string                   `json:"shipping_fee" binding:"required_if=ShippingFeeType 1" label:"统一运费"`    // 统一运费的值（计费方式为）
	SaleState       enum.MallGoodsSaleState  `json:"sale_state" label:"开售时间"`                                              // 销售状态
	SaleStartTime   string                   `json:"sale_start_time" label:"定时开售"`                                         // 定时上架时间
	CustomProperty  []struct {
		GroupName string `json:"group_name" label:"属性组别名称"`
		Index     int    `json:"index" label:"属性值索引"`
		Name      string `json:"name" label:"属性值"`
	} `json:"custom_property" label:"自定义参数"` // 自定义参数
	Spe                []Spe    `json:"spe" binding:"required,gt=0" label:"商品规格"`                     // 规格
	SpeImages          []string `json:"spe_images" binding:"required_if=IsSpeImageOn 1" label:"规格图片"` // 规格图片，与第一组规格名对应
	SpeImagesUrl       []string `json:"spe_images_url" label:"规格图片"`                                  // 规格图片，与第一组规格名对应
	Detail             []string `json:"detail" label:"商品详情"`                                          // 商品详情
	DetailUrl          []string `json:"detail_url" `                                                  // 商品详情
	MerchantId         int      `json:"merchant_id" `                                                 // 商品详情
	IsSpeImageOn       int      `json:"is_spe_image_on" default:"0" label:"是否开启规格图片"`                 // 是否要规格图
	IsSpeImageInDetail int      `json:"is_spe_image_in_detail" default:"0" label:"规格图片是否显示在详情中"`      // 规格图是否要显示在详情页
	ShippingTimeType   int      `json:"shipping_time_type" default:"3" label:"承诺发货时间"`                // 配送时间类型：1.当日发货 2.24小时内发货 3.48小时内发货 4自定义
	ShippingTimeCustom string   `json:"shipping_time_custom" label:"自动义发货时间"`
	PriceAddType       int      `json:"price_add_type" label:"加价方式(1:按金额加 2:按比例加)"`
	RatioBaseType      int      `json:"ratio_base_type" label:"比例基数方式(1:按进货价 2:按市场指导价 3:按利润空间"`
	AddPriceValue      string   `json:"add_price_value" label:"调整值(百分比或具体金额数值)"`
	PriceValueType     int      `json:"price_value_type" label:"价格数值设置(1:整数 2:小数点后两位)"`
	IsOpenAddPrice     int      `json:"is_open_add_price" label:"是否开启商品加价"`
	//云链的产品id（1688）
	CloudChainGoodsId string `json:"cloud_chain_goods_id"` //云链产品id
	MinOrderQuantity  string `json:"min_order_quantity"`   //最小购买数

}
type Spe struct {
	Name   string   `json:"name" label:"规格名"`   // 规格名
	Values []string `json:"values" label:"规格值"` // 规格值
}

type MallGoodsSku struct {
	SkuID     int               `json:"sku_id,omitempty" label:"skuID" copier:"-"` // sku_id 0或不传为新建
	GoodsID   int               `json:"goods_id,omitempty" label:"商品ID"`           // 0或不传为新建
	SkuCode   string            `json:"sku_code,omitempty" label:"sku編碼"`
	Price     string            `json:"price" binding:"required" label:"价格"` // 价格
	LinePrice string            `json:"line_price" binding:"required" label:"市场价"`
	Weight    string            `json:"weight" binding:"required" label:"重量"` // 重量
	Stock     int               `json:"stock" default:"0" label:"库存"`         // 库存
	Sku       []MallGoodsSkuSku `json:"sku" label:"规格组合"`                     // 规格组合

	//云链的产品属性
	SpecID              string `json:"spec_id"`
	CloudChainSkuId     string `json:"cloud_chain_sku_id"`
	CloudChainSkuImgUrl string `json:"cloud_chain_sku_img_url"`
}
type MallGoodsSkuSku struct {
	Name  string `json:"name" binding:"required" label:"规格名"`  // 规格名
	Value string `json:"value" binding:"required" label:"规格值"` // 规格值
}

type AddAdminGoodsWarehouse struct {
	Kind            string   `json:"kind" binding:"required"` //1、指定商品 2、全部商品 3、指定主營类目 4、指定供貨商
	GoodsId         []string `json:"goods_id"`
	MainCategoryIds []string `json:"main_category_ids"`
	MerchantIds     []string `json:"merchant_ids"`
	PriceAddType    string   `json:"price_add_type"`
	RatioBaseType   string   `json:"ratio_base_type"`
	PriceValueType  string   `json:"price_value_type"`
	AddPriceValue   string   `json:"add_price_value"`
}
